@@ -29,6 +29,7 @@
  */
 
 #include "util/omc_error.h"
+#include "util/omc_file.h"
 #include "simulation_data.h"
 #include "openmodelica_func.h"
 #include "simulation/solver/external_input.h"
@@ -291,6 +292,10 @@ int functionJacA(DATA* data, threadData_t *threadData, double* jac){
   ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[index]);
   unsigned int i,j,k;
   k = 0;
+  if (jacobian->constantEqns != NULL) {
+    jacobian->constantEqns(data, threadData, jacobian, NULL);
+  }
+
   for(i=0; i < jacobian->sizeCols; i++)
   {
     jacobian->seedVars[i] = 1.0;
@@ -334,6 +339,10 @@ int functionJacB(DATA* data, threadData_t *threadData, double* jac){
 
   unsigned int i,j,k;
   k = 0;
+  if (jacobian->constantEqns != NULL) {
+    jacobian->constantEqns(data, threadData, jacobian, NULL);
+  }
+
   for(i=0; i < jacobian->sizeCols; i++)
   {
     jacobian->seedVars[i] = 1.0;
@@ -375,6 +384,10 @@ int functionJacC(DATA* data, threadData_t *threadData, double* jac){
   ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[index]);
   unsigned int i,j,k;
   k = 0;
+  if (jacobian->constantEqns != NULL) {
+    jacobian->constantEqns(data, threadData, jacobian, NULL);
+  }
+
   for(i=0; i < jacobian->sizeCols; i++)
   {
     jacobian->seedVars[i] = 1.0;
@@ -414,6 +427,10 @@ int functionJacD(DATA* data, threadData_t *threadData, double* jac){
   ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[index]);
   unsigned int i,j,k;
   k = 0;
+  if (jacobian->constantEqns != NULL) {
+    jacobian->constantEqns(data, threadData, jacobian, NULL);
+  }
+
   for(i=0; i < jacobian->sizeCols; i++)
   {
     jacobian->seedVars[i] = 1.0;
@@ -604,8 +621,7 @@ int linearize(DATA* data, threadData_t *threadData)
     }
 #endif
 
-
-    FILE *fout = fopen(filename.c_str(),"wb");
+    FILE *fout = omc_fopen(filename.c_str(),"wb");
     assertStreamPrint(threadData,0!=fout,"Cannot open File %s",filename.c_str());
     if(do_data_recovery > 0){
         fprintf(fout, data->callback->linear_model_datarecovery_frame(), strX.c_str(), strU.c_str(), strZ0.c_str(), strA.c_str(), strB.c_str(), strC.c_str(), strD.c_str(), strCz.c_str(), strDz.c_str());
