@@ -35,8 +35,11 @@
 #include <string.h>
 
 #include "model_help.h"
-#include "util/omc_error.h"
-#include "util/rtclock.h"
+#include "../../util/omc_error.h"
+#include "../../util/parallel_helper.h"
+#include "../../util/jacobian_util.h"
+#include "../../util/rtclock.h"
+#include "nonlinearSystem.h"
 #include "linearSystem.h"
 #include "linearSolverLapack.h"
 #if !defined(OMC_MINIMAL_RUNTIME)
@@ -45,7 +48,7 @@
 #include "linearSolverUmfpack.h"
 #endif
 #include "linearSolverTotalPivot.h"
-#include "simulation/simulation_info_json.h"
+#include "../simulation_info_json.h"
 
 static void setAElement(int row, int col, double value, int nth, void *data, threadData_t *);
 static void setAElementLis(int row, int col, double value, int nth, void *data, threadData_t *);
@@ -56,15 +59,6 @@ static void setBElementLis(int row, double value, void *data, threadData_t*);
 
 int check_linear_solution(DATA *data, int printFailingSystems, int sysNumber);
 
-/* Data structure for the default solver
- * where two different solverData for lapack and
- * totalpivot as fallback
- */
-struct dataLapackAndTotalPivot{
-  void* lapackData;
-  void* totalpivotData;
-};
-
 /*! \fn int initializeLinearSystems(DATA *data)
  *
  *  This function allocates memory for all linear systems.
@@ -76,22 +70,38 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
   TRACE_PUSH
   int i, nnz;
   int size;
+  int res;
+  unsigned int j, maxNumberThreads;
   LINEAR_SYSTEM_DATA *linsys = data->simulationInfo->linearSystemData;
-  struct dataLapackAndTotalPivot *defaultSolverData;
 
-  infoStreamPrint(LOG_LS_V, 1, "initialize linear system solvers");
+  maxNumberThreads = omc_get_max_threads();
+
+  infoStreamPrint(LOG_LS, 1, "initialize linear system solvers");
+  infoStreamPrint(LOG_LS, 0, "%ld linear systems", data->modelData->nLinearSystems);
+
+  if (LSS_DEFAULT == data->simulationInfo->lssMethod) {
+#ifdef WITH_UMFPACK
+    data->simulationInfo->lssMethod = LSS_KLU;
+#elif !defined(OMC_MINIMAL_RUNTIME)
+    data->simulationInfo->lssMethod = LSS_LIS;
+#endif
+  }
 
   for(i=0; i<data->modelData->nLinearSystems; ++i)
   {
+    res = allocLinSystThreadData(&(linsys[i]));
+    assertStreamPrint(threadData, !res ,"Out of memory");
+
     size = linsys[i].size;
     nnz = linsys[i].nnz;
-
     linsys[i].totalTime = 0;
     linsys[i].failed = 0;
 
     /* allocate system data */
-    linsys[i].x = (double*) malloc(size*sizeof(double));
-    linsys[i].b = (double*) malloc(size*sizeof(double));
+    for (j=0; j<maxNumberThreads; ++j)
+    {
+      linsys[i].parDynamicData[j].b = (double*) malloc(size*sizeof(double));
+    }
 
     /* check if analytical jacobian is created */
     if (1 == linsys[i].method)
@@ -100,19 +110,40 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
       {
         assertStreamPrint(threadData, 0 != linsys[i].analyticalJacobianColumn, "jacobian function pointer is invalid" );
       }
-      if(linsys[i].initialAnalyticalJacobian(data, threadData))
+      ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[linsys[i].jacobianIndex]);
+      if(linsys[i].initialAnalyticalJacobian(data, threadData, jacobian))
       {
         linsys[i].jacobianIndex = -1;
         throwStreamPrint(threadData, "Failed to initialize the jacobian for torn linear system %d.", (int)linsys[i].equationIndex);
       }
-      nnz = data->simulationInfo->analyticJacobians[linsys[i].jacobianIndex].sparsePattern.numberOfNoneZeros;
+      nnz = jacobian->sparsePattern->numberOfNoneZeros;
       linsys[i].nnz = nnz;
+
+#ifdef USE_PARJAC
+      /* Allocate jacobian for parDynamicData */
+      for (j=0; j<maxNumberThreads; ++j)
+      {
+        // ToDo Simplify this. Only have one location for jacobian
+        linsys[i].parDynamicData[j].jacobian = (ANALYTIC_JACOBIAN*) malloc(sizeof(ANALYTIC_JACOBIAN));
+        //linsys[i].initialAnalyticalJacobian(data, threadData, linsys[i].parDynamicData[j].jacobian);
+        linsys[i].parDynamicData[j].jacobian->sizeCols = jacobian->sizeCols;
+        linsys[i].parDynamicData[j].jacobian->sizeRows = jacobian->sizeRows;
+        linsys[i].parDynamicData[j].jacobian->sizeTmpVars = jacobian->sizeTmpVars;
+        linsys[i].parDynamicData[j].jacobian->seedVars = (modelica_real*) calloc(jacobian->sizeCols, sizeof(modelica_real));
+        linsys[i].parDynamicData[j].jacobian->resultVars = (modelica_real*) calloc(jacobian->sizeRows, sizeof(modelica_real));
+        linsys[i].parDynamicData[j].jacobian->tmpVars = (modelica_real*) calloc(jacobian->sizeTmpVars, sizeof(modelica_real));
+        linsys[i].parDynamicData[j].jacobian->constantEqns = jacobian->constantEqns;
+        linsys[i].parDynamicData[j].jacobian->sparsePattern = jacobian->sparsePattern;
+      }
+#else
+      linsys[i].parDynamicData[0].jacobian = jacobian;
+#endif
     }
 
     if(nnz/(double)(size*size)<=linearSparseSolverMaxDensity && size>=linearSparseSolverMinSize)
     {
       linsys[i].useSparseSolver = 1;
-      infoStreamPrint(LOG_STDOUT, 0, "Using sparse solver for linear system %d,\nbecause density of %.2f remains under threshold of %.2f and size of %d exceeds threshold of %d.\nThe maximum density and the minimal system size for using sparse solvers can be specified\nusing the runtime flags '<-lssMaxDensity=value>' and '<-lssMinSize=value>'.", i, nnz/(double)(size*size), linearSparseSolverMaxDensity, size, linearSparseSolverMinSize);
+      infoStreamPrint(LOG_STDOUT, 0, "Using sparse solver for linear system %d,\nbecause density of %.3f remains under threshold of %.3f and size of %d exceeds threshold of %d.\nThe maximum density and the minimal system size for using sparse solvers can be specified\nusing the runtime flags '<-lssMaxDensity=value>' and '<-lssMinSize=value>'.", i, nnz/(double)(size*size), linearSparseSolverMaxDensity, size, linearSparseSolverMinSize);
     }
 
     /* allocate more system data */
@@ -128,62 +159,96 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
     {
       switch(data->simulationInfo->lssMethod)
       {
-    #if !defined(OMC_MINIMAL_RUNTIME)
-      case LSS_LIS:
-        linsys[i].setAElement = setAElementLis;
-        linsys[i].setBElement = setBElementLis;
-        allocateLisData(size, size, nnz, &linsys[i].solverData);
-        break;
-    #endif
     #ifdef WITH_UMFPACK
       case LSS_UMFPACK:
         linsys[i].setAElement = setAElementUmfpack;
         linsys[i].setBElement = setBElement;
-        allocateUmfPackData(size, size, nnz, &linsys[i].solverData);
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          allocateUmfPackData(size, size, nnz, linsys[i].parDynamicData[j].solverData);
+        }
         break;
       case LSS_KLU:
         linsys[i].setAElement = setAElementKlu;
         linsys[i].setBElement = setBElement;
-        allocateKluData(size, size, nnz, &linsys[i].solverData);
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          allocateKluData(size, size, nnz, linsys[i].parDynamicData[j].solverData);
+        }
         break;
     #else
+      case LSS_KLU:
       case LSS_UMFPACK:
-        throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
+        throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use klu or umfpack please compile OMC with UMFPACK.");
         break;
     #endif
-
+    #if !defined(OMC_MINIMAL_RUNTIME)
+      case LSS_LIS:
+        linsys[i].setAElement = setAElementLis;
+        linsys[i].setBElement = setBElementLis;
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          allocateLisData(size, size, nnz, linsys[i].parDynamicData[j].solverData);
+        }
+        break;
+    #else
+      case LSS_LIS_NOT_AVAILABLE:
+        throwStreamPrint(threadData, "OMC is compiled without sparse linear solver Lis.");
+        break;
+    #endif
+    #if defined(OMC_MINIMAL_RUNTIME) && !defined(WITH_UMFPACK)
+      case LSS_DEFAULT:
+        {
+          int indexes[2] = {1, linsys[i].equationIndex};
+          infoStreamPrintWithEquationIndexes(LOG_STDOUT, 0, indexes, "The simulation runtime does not have access to sparse solvers. Defaulting to a dense linear system solver instead.");
+          linsys[i].useSparseSolver = 0;
+          break;
+        }
+    #endif
       default:
-        throwStreamPrint(threadData, "unrecognized linear solver");
+        throwStreamPrint(threadData, "unrecognized sparse linear solver (%d)", data->simulationInfo->lssMethod);
       }
     }
-
-    else{
+    if(linsys[i].useSparseSolver == 0) { /* Not an else-statement because there might not be a sparse linear solver available */
     switch(data->simulationInfo->lsMethod)
     {
       case LS_LAPACK:
-        linsys[i].A = (double*) malloc(size*size*sizeof(double));
         linsys[i].setAElement = setAElement;
         linsys[i].setBElement = setBElement;
-        allocateLapackData(size, &linsys[i].solverData);
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          linsys[i].parDynamicData[j].A = (double*) malloc(size*size*sizeof(double));
+          allocateLapackData(size, linsys[i].parDynamicData[j].solverData);
+        }
+
         break;
 
     #if !defined(OMC_MINIMAL_RUNTIME)
       case LS_LIS:
         linsys[i].setAElement = setAElementLis;
         linsys[i].setBElement = setBElementLis;
-        allocateLisData(size, size, nnz, &linsys[i].solverData);
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          allocateLisData(size, size, nnz, linsys[i].parDynamicData[j].solverData);
+        }
         break;
     #endif
     #ifdef WITH_UMFPACK
       case LS_UMFPACK:
         linsys[i].setAElement = setAElementUmfpack;
         linsys[i].setBElement = setBElement;
-        allocateUmfPackData(size, size, nnz, &linsys[i].solverData);
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          allocateUmfPackData(size, size, nnz, linsys[i].parDynamicData[j].solverData);
+        }
         break;
       case LS_KLU:
         linsys[i].setAElement = setAElementKlu;
         linsys[i].setBElement = setBElement;
-        allocateKluData(size, size, nnz, &linsys[i].solverData);
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          allocateKluData(size, size, nnz, linsys[i].parDynamicData[j].solverData);
+        }
         break;
     #else
       case LS_UMFPACK:
@@ -192,36 +257,67 @@ int initializeLinearSystems(DATA *data, threadData_t *threadData)
     #endif
 
       case LS_TOTALPIVOT:
-        linsys[i].A = (double*) malloc(size*size*sizeof(double));
         linsys[i].setAElement = setAElement;
         linsys[i].setBElement = setBElement;
-        allocateTotalPivotData(size, &(linsys[i].solverData));
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          linsys[i].parDynamicData[j].A = (double*) malloc(size*size*sizeof(double));
+          allocateTotalPivotData(size, linsys[i].parDynamicData[j].solverData);
+        }
         break;
 
       case LS_DEFAULT:
-        defaultSolverData = (struct dataLapackAndTotalPivot*) malloc(sizeof(struct dataLapackAndTotalPivot));
-        linsys[i].A = (double*) malloc(size*size*sizeof(double));
         linsys[i].setAElement = setAElement;
         linsys[i].setBElement = setBElement;
-
-        allocateLapackData(size, &(defaultSolverData->lapackData));
-        allocateTotalPivotData(size, &(defaultSolverData->totalpivotData));
-
-        linsys[i].solverData = (void*) defaultSolverData;
+        for (j=0; j<maxNumberThreads; ++j)
+        {
+          linsys[i].parDynamicData[j].A = (double*) malloc(size*size*sizeof(double));
+          allocateLapackData(size, linsys[i].parDynamicData[j].solverData);
+          allocateTotalPivotData(size, linsys[i].parDynamicData[j].solverData);
+        }
         break;
 
       default:
-        throwStreamPrint(threadData, "unrecognized linear solver");
+        throwStreamPrint(threadData, "unrecognized dense linear solver (%d)", data->simulationInfo->lsMethod);
       }
     }
   }
 
-  messageClose(LOG_LS_V);
+  messageClose(LOG_LS);
 
   TRACE_POP
   return 0;
 }
 
+
+/**
+ * \brief Allocates memory for thread safe data struct for linear systems.
+ *
+ *  Allocates memory array of numberOfThreads many LINEAR_SYSTEM_THREAD_DATA structs.
+ *  Returns -1 if out of memroy.
+ *
+ *  \param [in/out]   linsys
+ */
+int allocLinSystThreadData(LINEAR_SYSTEM_DATA *linsys)
+{
+  linsys->parDynamicData = (LINEAR_SYSTEM_THREAD_DATA*) malloc(omc_get_max_threads()*sizeof(LINEAR_SYSTEM_THREAD_DATA));
+  if (!linsys->parDynamicData)
+    return -1;
+  return 0;
+}
+
+/**
+ * \brief Frees memory allocated with `allocLinSystThreadData`
+ *
+ *  Frees complete array of numberOfThreads many LINEAR_SYSTEM_THREAD_DATA structs.
+ *
+ *  \param [in/out]   linsys
+ */
+void freeLinSystThreadData(LINEAR_SYSTEM_DATA *linsys)
+{
+  free(linsys->parDynamicData);
+}
+
 /*! \fn int updateStaticDataOfLinearSystems(DATA *data)
  *
  *  This function allocates memory for all linear systems.
@@ -263,6 +359,7 @@ void printLinearSystemSolvingStatistics(DATA *data, int sysNumber, int logLevel)
                                (((double) linsys[sysNumber].nnz) / ((double)(linsys[sysNumber].size*linsys[sysNumber].size)))*100 );
   infoStreamPrint(logLevel, 0, " number of calls                : %ld", linsys[sysNumber].numberOfCall);
   infoStreamPrint(logLevel, 0, " average time per call          : %g", linsys[sysNumber].totalTime/linsys[sysNumber].numberOfCall);
+  infoStreamPrint(logLevel, 0, " time of jacobian evaluations   : %g", linsys[sysNumber].jacobianTime);
   infoStreamPrint(logLevel, 0, " total time                     : %g", linsys[sysNumber].totalTime);
   messageClose(logLevel);
 }
@@ -276,35 +373,67 @@ void printLinearSystemSolvingStatistics(DATA *data, int sysNumber, int logLevel)
 int freeLinearSystems(DATA *data, threadData_t *threadData)
 {
   TRACE_PUSH
-  int i;
+  int i,j;
   LINEAR_SYSTEM_DATA* linsys = data->simulationInfo->linearSystemData;
 
   infoStreamPrint(LOG_LS_V, 1, "free linear system solvers");
   for(i=0; i<data->modelData->nLinearSystems; ++i)
   {
     /* free system and solver data */
-    free(linsys[i].x);
-    free(linsys[i].b);
+    for (j=0; j<omc_get_max_threads(); ++j)
+    {
+      free(linsys[i].parDynamicData[j].b);
+    }
     free(linsys[i].nominal);
     free(linsys[i].min);
     free(linsys[i].max);
 
+    /* ToDo Implement unique function to free a ANALYTIC_JACOBIAN */
+    if (1 == linsys[i].method) {
+      ANALYTIC_JACOBIAN* jacobian = &(data->simulationInfo->analyticJacobians[linsys[i].jacobianIndex]);
+      freeAnalyticJacobian(jacobian);
+      /* Note: The Jacobian of data->simulationInfo itself will be free later. */
+
+#ifdef USE_PARJAC
+      for (j=0; j<omc_get_max_threads(); ++j) {
+        // Note: We cannot use neither freeAnalyticJacobian() nor freeSparsePattern()
+        //       since the sparsePattern points to data->simulationInfo->analyticJacobians[linsys[i].jacobianIndex]
+        //       which is free some lines above (and are invalid pointers at this point). Thus, free
+        //       what is left.
+        free(linsys[i].parDynamicData[j].jacobian->seedVars); linsys[i].parDynamicData[j].jacobian->seedVars = NULL;
+        free(linsys[i].parDynamicData[j].jacobian->resultVars); linsys[i].parDynamicData[j].jacobian->resultVars = NULL;
+        free(linsys[i].parDynamicData[j].jacobian->tmpVars); linsys[i].parDynamicData[j].jacobian->tmpVars = NULL;
+        linsys[i].parDynamicData[j].jacobian->sparsePattern = NULL;
+        free(linsys[i].parDynamicData[j].jacobian); linsys[i].parDynamicData[j].jacobian = NULL;
+      }
+#endif
+    }
+
     if(linsys[i].useSparseSolver == 1)
     {
       switch(data->simulationInfo->lssMethod)
       {
     #if !defined(OMC_MINIMAL_RUNTIME)
       case LSS_LIS:
-        freeLisData(&linsys[i].solverData);
+        for (j=0; j<omc_get_max_threads(); ++j)
+        {
+          freeLisData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
     #endif
 
     #ifdef WITH_UMFPACK
       case LSS_UMFPACK:
-        freeUmfPackData(&linsys[i].solverData);
+        for (j=0; j<omc_get_max_threads(); ++j)
+        {
+          freeUmfPackData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
       case LSS_KLU:
-        freeKluData(&linsys[i].solverData);
+        for (j=0; j<omc_get_max_threads(); ++j)
+        {
+          freeKluData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
     #else
       case LSS_UMFPACK:
@@ -313,30 +442,38 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
     #endif
 
       default:
-        throwStreamPrint(threadData, "unrecognized linear solver");
+        throwStreamPrint(threadData, "unrecognized sparse linear solver (%d)", data->simulationInfo->lssMethod);
       }
     }
-
-    else{
-      switch(data->simulationInfo->lsMethod)
-      {
+    else { // useSparseSolver == 0
+      switch(data->simulationInfo->lsMethod) {
       case LS_LAPACK:
-        freeLapackData(&linsys[i].solverData);
-        free(linsys[i].A);
+        for (j=0; j<omc_get_max_threads(); ++j) {
+          free(linsys[i].parDynamicData[j].A);
+          freeLapackData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
 
   #if !defined(OMC_MINIMAL_RUNTIME)
       case LS_LIS:
-        freeLisData(&linsys[i].solverData);
+        for (j=0; j<omc_get_max_threads(); ++j) {
+          freeLisData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
   #endif
 
   #ifdef WITH_UMFPACK
       case LS_UMFPACK:
-        freeUmfPackData(&linsys[i].solverData);
+        for (j=0; j<omc_get_max_threads(); ++j)
+        {
+          freeUmfPackData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
       case LS_KLU:
-        freeKluData(&linsys[i].solverData);
+        for (j=0; j<omc_get_max_threads(); ++j)
+        {
+          freeKluData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
   #else
       case LS_UMFPACK:
@@ -345,22 +482,28 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
   #endif
 
       case LS_TOTALPIVOT:
-        free(linsys[i].A);
-        freeTotalPivotData(&(linsys[i].solverData));
+        for (j=0; j<omc_get_max_threads(); ++j)
+        {
+          free(linsys[i].parDynamicData[j].A);
+          freeTotalPivotData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
 
       case LS_DEFAULT:
-        free(linsys[i].A);
-        freeLapackData(&((struct dataLapackAndTotalPivot*) linsys[i].solverData)->lapackData);
-        freeTotalPivotData(&((struct dataLapackAndTotalPivot*) linsys[i].solverData)->totalpivotData);
+        for (j=0; j<omc_get_max_threads(); ++j) {
+          free(linsys[i].parDynamicData[j].A);
+          freeLapackData(linsys[i].parDynamicData[j].solverData);
+          freeTotalPivotData(linsys[i].parDynamicData[j].solverData);
+        }
         break;
 
       default:
-        throwStreamPrint(threadData, "unrecognized linear solver");
+        throwStreamPrint(threadData, "unrecognized dense linear solver (data->simulationInfo->lsMethod)");
       }
     }
 
-    free(linsys[i].solverData);
+    /* Free linear system thread data */
+    freeLinSystThreadData(&(linsys[i]));
   }
 
   messageClose(LOG_LS_V);
@@ -371,19 +514,18 @@ int freeLinearSystems(DATA *data, threadData_t *threadData)
 
 /*! \fn solve linear system
  *
- *  \param [in]  [data]
- *         [in]  [sysNumber] index of corresponding linear System
- *
- *  \author wbraun
+ *  \param  [ref]     data
+ *          [ref]     threadData
+ *          [in]      sysNumber   index of corresponding linear System
+ *          [in/out]  aux_x       Auxiliary vector with values for x. Contains solution on output.
  */
-int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber)
+int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber, double* aux_x)
 {
   TRACE_PUSH
-  int success;
   int retVal;
+  int success;
   int logLevel;
   LINEAR_SYSTEM_DATA* linsys = &(data->simulationInfo->linearSystemData[sysNumber]);
-  struct dataLapackAndTotalPivot *defaultSolverData;
 
   rt_ext_tp_tick(&(linsys->totalTimeClock));
 
@@ -396,15 +538,19 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber)
     {
   #if !defined(OMC_MINIMAL_RUNTIME)
     case LSS_LIS:
-      success = solveLis(data, threadData, sysNumber);
+      success = solveLis(data, threadData, sysNumber, aux_x);
+      break;
+  #else
+    case LSS_LIS_NOT_AVAILABLE:
+      throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
       break;
   #endif
   #ifdef WITH_UMFPACK
     case LSS_KLU:
-      success = solveKlu(data, threadData, sysNumber);
+      success = solveKlu(data, threadData, sysNumber, aux_x);
       break;
     case LSS_UMFPACK:
-      success = solveUmfPack(data, threadData, sysNumber);
+      success = solveUmfPack(data, threadData, sysNumber, aux_x);
       if (!success && linsys->strictTearingFunctionCall != NULL){
         debugString(LOG_DT, "Solving the casual tearing set failed! Now the strict tearing set is used.");
         success = linsys->strictTearingFunctionCall(data, threadData);
@@ -412,13 +558,13 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber)
       }
       break;
   #else
+    case LSS_KLU:
     case LSS_UMFPACK:
       throwStreamPrint(threadData, "OMC is compiled without UMFPACK, if you want use umfpack please compile OMC with UMFPACK.");
       break;
   #endif
-
     default:
-      throwStreamPrint(threadData, "unrecognized linear solver");
+      throwStreamPrint(threadData, "unrecognized sparse linear solver (%d)", data->simulationInfo->lssMethod);
     }
   }
 
@@ -426,20 +572,20 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber)
     switch(data->simulationInfo->lsMethod)
     {
     case LS_LAPACK:
-      success = solveLapack(data, threadData, sysNumber);
+      success = solveLapack(data, threadData, sysNumber, aux_x);
       break;
 
   #if !defined(OMC_MINIMAL_RUNTIME)
     case LS_LIS:
-      success = solveLis(data, threadData, sysNumber);
+      success = solveLis(data, threadData, sysNumber, aux_x);
       break;
   #endif
   #ifdef WITH_UMFPACK
     case LS_KLU:
-      success = solveKlu(data, threadData, sysNumber);
+      success = solveKlu(data, threadData, sysNumber, aux_x);
       break;
     case LS_UMFPACK:
-      success = solveUmfPack(data, threadData, sysNumber);
+      success = solveUmfPack(data, threadData, sysNumber, aux_x);
       if (!success && linsys->strictTearingFunctionCall != NULL){
         debugString(LOG_DT, "Solving the casual tearing set failed! Now the strict tearing set is used.");
         success = linsys->strictTearingFunctionCall(data, threadData);
@@ -453,14 +599,11 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber)
   #endif
 
     case LS_TOTALPIVOT:
-      success = solveTotalPivot(data, threadData, sysNumber);
+      success = solveTotalPivot(data, threadData, sysNumber, aux_x);
       break;
 
     case LS_DEFAULT:
-      defaultSolverData = linsys->solverData;
-      linsys->solverData = defaultSolverData->lapackData;
-
-      success = solveLapack(data, threadData, sysNumber);
+      success = solveLapack(data, threadData, sysNumber, aux_x);
 
       /* check if solution process was successful, if not use alternative tearing set if available (dynamic tearing)*/
       if (!success && linsys->strictTearingFunctionCall != NULL){
@@ -470,7 +613,7 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber)
           success=2;
           linsys->failed = 0;
         }
-        else{
+        else {
           linsys->failed = 1;
         }
       }
@@ -483,18 +626,16 @@ int solve_linear_system(DATA *data, threadData_t *threadData, int sysNumber)
           logLevel = LOG_STDOUT;
         }
         warningStreamPrint(logLevel, 0, "The default linear solver fails, the fallback solver with total pivoting is started at time %f. That might raise performance issues, for more information use -lv LOG_LS.", data->localData[0]->timeValue);
-        linsys->solverData = defaultSolverData->totalpivotData;
-        success = solveTotalPivot(data, threadData, sysNumber);
+        success = solveTotalPivot(data, threadData, sysNumber, aux_x);
         linsys->failed = 1;
-      }else{
+      } else {
         linsys->failed = 0;
       }
       }
-      linsys->solverData = defaultSolverData;
       break;
 
     default:
-      throwStreamPrint(threadData, "unrecognized linear solver");
+      throwStreamPrint(threadData, "unrecognized dense linear solver (%d)", data->simulationInfo->lsMethod);
     }
   }
   linsys->solved = success;
@@ -562,7 +703,11 @@ int check_linear_solution(DATA *data, int printFailingSystems, int sysNumber)
       TRACE_POP
       return 1;
     }
+#ifdef USE_PARJAC
+    warningStreamPrintWithEquationIndexes(LOG_STDOUT, 1, indexes, "Thread %u: Solving linear system %d fails at time %g. For more information use -lv LOG_LS.", omc_get_thread_num(), index, data->localData[0]->timeValue);
+#else
     warningStreamPrintWithEquationIndexes(LOG_STDOUT, 1, indexes, "Solving linear system %d fails at time %g. For more information use -lv LOG_LS.", index, data->localData[0]->timeValue);
+#endif
 
     for(j=0; j<modelInfoGetEquation(&data->modelData->modelDataXml, (linsys[i]).equationIndex).numVar; ++j) {
       int done=0;
@@ -614,7 +759,7 @@ int check_linear_solution(DATA *data, int printFailingSystems, int sysNumber)
 static void setAElement(int row, int col, double value, int nth, void *data, threadData_t *threadData)
 {
   LINEAR_SYSTEM_DATA* linsys = (LINEAR_SYSTEM_DATA*) data;
-  linsys->A[row + col * linsys->size] = value;
+  linsys->parDynamicData[omc_get_thread_num()].A[row + col * linsys->size] = value;
 }
 
 /*! \fn setBElement
@@ -627,21 +772,21 @@ static void setAElement(int row, int col, double value, int nth, void *data, thr
 static void setBElement(int row, double value, void *data, threadData_t *threadData)
 {
   LINEAR_SYSTEM_DATA* linsys = (LINEAR_SYSTEM_DATA*) data;
-  linsys->b[row] = value;
+  linsys->parDynamicData[omc_get_thread_num()].b[row] = value;
 }
 
 #if !defined(OMC_MINIMAL_RUNTIME)
 static void setAElementLis(int row, int col, double value, int nth, void *data, threadData_t *threadData)
 {
   LINEAR_SYSTEM_DATA* linsys = (LINEAR_SYSTEM_DATA*) data;
-  DATA_LIS* sData = (DATA_LIS*) linsys->solverData;
+  DATA_LIS* sData = (DATA_LIS*) linsys->parDynamicData[omc_get_thread_num()].solverData[0];
   lis_matrix_set_value(LIS_INS_VALUE, row, col, value, sData->A);
 }
 
 static void setBElementLis(int row, double value, void *data, threadData_t *threadData)
 {
   LINEAR_SYSTEM_DATA* linsys = (LINEAR_SYSTEM_DATA*) data;
-  DATA_LIS* sData = (DATA_LIS*) linsys->solverData;
+  DATA_LIS* sData = (DATA_LIS*) linsys->parDynamicData[omc_get_thread_num()].solverData[0];
   lis_vector_set_value(LIS_INS_VALUE, row, value, sData->b);
 }
 #endif
@@ -650,11 +795,11 @@ static void setBElementLis(int row, double value, void *data, threadData_t *thre
 static void setAElementUmfpack(int row, int col, double value, int nth, void *data, threadData_t *threadData)
 {
   LINEAR_SYSTEM_DATA* linSys = (LINEAR_SYSTEM_DATA*) data;
-  DATA_UMFPACK* sData = (DATA_UMFPACK*) linSys->solverData;
+  DATA_UMFPACK* sData = (DATA_UMFPACK*) linSys->parDynamicData[omc_get_thread_num()].solverData[0];
 
   infoStreamPrint(LOG_LS_V, 0, " set %d. -> (%d,%d) = %f", nth, row, col, value);
-  if (row > 0){
-    if (sData->Ap[row] == 0){
+  if (row > 0) {
+    if (sData->Ap[row] == 0) {
       sData->Ap[row] = nth;
     }
   }
@@ -662,13 +807,14 @@ static void setAElementUmfpack(int row, int col, double value, int nth, void *da
   sData->Ai[nth] = col;
   sData->Ax[nth] = value;
 }
+
 static void setAElementKlu(int row, int col, double value, int nth, void *data, threadData_t *threadData)
 {
   LINEAR_SYSTEM_DATA* linSys = (LINEAR_SYSTEM_DATA*) data;
-  DATA_KLU* sData = (DATA_KLU*) linSys->solverData;
+  DATA_KLU* sData = (DATA_KLU*) linSys->parDynamicData[omc_get_thread_num()].solverData[0];
 
-  if (row > 0){
-    if (sData->Ap[row] == 0){
+  if (row > 0) {
+    if (sData->Ap[row] == 0) {
       sData->Ap[row] = nth;
     }
   }
@@ -676,4 +822,5 @@ static void setAElementKlu(int row, int col, double value, int nth, void *data,
   sData->Ai[nth] = col;
   sData->Ax[nth] = value;
 }
+
 #endif
