@@ -43,6 +43,7 @@
 #include "ida_solver.h"
 #include "delay.h"
 #include "events.h"
+#include "util/parallel_helper.h"
 #include "util/varinfo.h"
 #include "radau.h"
 #include "model_help.h"
@@ -360,6 +361,7 @@ int freeSolverData(DATA* data, SOLVER_INFO* solverInfo)
   int retValue = 0;
   int i;
 
+  freeList(solverInfo->eventLst);
   /* free solver statistics */
   free(solverInfo->solverStats);
   free(solverInfo->solverStatsTmp);
@@ -439,7 +441,6 @@ int initializeModel(DATA* data, threadData_t *threadData, const char* init_initM
   {
     rt_accumulate(SIM_TIMER_PREINIT);
     rt_tick(SIM_TIMER_INIT);
-    rt_tick(SIM_TIMER_TOTAL);
   }
 
   copyStartValuestoInitValues(data);
@@ -465,6 +466,13 @@ int initializeModel(DATA* data, threadData_t *threadData, const char* init_initM
       simInfo->stopTime = simInfo->startTime;
       retValue = -1;
     }
+    if (!retValue)
+    {
+      if (data->simulationInfo->homotopySteps == 0)
+        infoStreamPrint(LOG_SUCCESS, 0, "The initialization finished successfully without homotopy method.");
+      else
+        infoStreamPrint(LOG_SUCCESS, 0, "The initialization finished successfully with %d homotopy steps.", data->simulationInfo->homotopySteps);
+    }
 
     success = 1;
 #if !defined(OMC_EMCC)
@@ -506,6 +514,7 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
 
   int retValue = 0;
   int ui;
+  double t, total100;
 
   SIMULATION_INFO *simInfo = data->simulationInfo;
 
@@ -539,22 +548,23 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
 
     infoStreamPrint(LOG_STATS, 1, "### STATISTICS ###");
 
+    total100 = rt_accumulated(SIM_TIMER_TOTAL)/100.0;
+
     infoStreamPrint(LOG_STATS, 1, "timer");
     infoStreamPrint(LOG_STATS, 0, "%12gs          reading init.xml", rt_accumulated(SIM_TIMER_INIT_XML));
     infoStreamPrint(LOG_STATS, 0, "%12gs          reading info.xml", rt_accumulated(SIM_TIMER_INFO_XML));
-    infoStreamPrint(LOG_STATS, 0, "%12gs          pre-initialization", rt_accumulated(SIM_TIMER_PREINIT));
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] initialization", rt_accumulated(SIM_TIMER_INIT), rt_accumulated(SIM_TIMER_INIT)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] steps", rt_accumulated(SIM_TIMER_STEP), rt_accumulated(SIM_TIMER_STEP)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] creating output-file", rt_accumulated(SIM_TIMER_OUTPUT), rt_accumulated(SIM_TIMER_OUTPUT)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] event-handling", rt_accumulated(SIM_TIMER_EVENT), rt_accumulated(SIM_TIMER_EVENT)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] overhead", rt_accumulated(SIM_TIMER_OVERHEAD), rt_accumulated(SIM_TIMER_OVERHEAD)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] pre-initialization", rt_accumulated(SIM_TIMER_PREINIT), rt_accumulated(SIM_TIMER_PREINIT)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] initialization", rt_accumulated(SIM_TIMER_INIT), rt_accumulated(SIM_TIMER_INIT)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] steps", rt_accumulated(SIM_TIMER_STEP), rt_accumulated(SIM_TIMER_STEP)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] solver (excl. callbacks)", rt_accumulated(SIM_TIMER_SOLVER), rt_accumulated(SIM_TIMER_SOLVER)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] creating output-file", rt_accumulated(SIM_TIMER_OUTPUT), rt_accumulated(SIM_TIMER_OUTPUT)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] event-handling", rt_accumulated(SIM_TIMER_EVENT), rt_accumulated(SIM_TIMER_EVENT)/total100);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] overhead", rt_accumulated(SIM_TIMER_OVERHEAD), rt_accumulated(SIM_TIMER_OVERHEAD)/total100);
 
-    if(S_OPTIMIZATION != solverInfo->solverMethod)
-      infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] simulation", rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT), (rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT))/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
-    else
-      infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] optimization", rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT), (rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT))/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
+    t = rt_accumulated(SIM_TIMER_TOTAL)-rt_accumulated(SIM_TIMER_OVERHEAD)-rt_accumulated(SIM_TIMER_EVENT)-rt_accumulated(SIM_TIMER_OUTPUT)-rt_accumulated(SIM_TIMER_STEP)-rt_accumulated(SIM_TIMER_INIT)-rt_accumulated(SIM_TIMER_PREINIT)-rt_accumulated(SIM_TIMER_SOLVER);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] %s", t, t/total100, S_OPTIMIZATION == solverInfo->solverMethod ? "optimization" : "simulation");
 
-    infoStreamPrint(LOG_STATS, 0, "%12gs [%5.1f%%] total", rt_accumulated(SIM_TIMER_TOTAL), rt_accumulated(SIM_TIMER_TOTAL)/rt_accumulated(SIM_TIMER_TOTAL)*100.0);
+    infoStreamPrint(LOG_STATS, 0, "%12gs [100.0%%] total", rt_accumulated(SIM_TIMER_TOTAL));
     messageClose(LOG_STATS);
 
     infoStreamPrint(LOG_STATS, 1, "events");
@@ -574,27 +584,61 @@ int finishSimulation(DATA* data, threadData_t *threadData, SOLVER_INFO* solverIn
 
       infoStreamPrint(LOG_STATS, 1, "solver: %s", SOLVER_METHOD_NAME[solverInfo->solverMethod]);
       infoStreamPrint(LOG_STATS, 0, "%5d steps taken", solverInfo->solverStats[0]);
+
       infoStreamPrint(LOG_STATS, 0, "%5d calls of functionODE", solverInfo->solverStats[1]);
+
       infoStreamPrint(LOG_STATS, 0, "%5d evaluations of jacobian", solverInfo->solverStats[2]);
+
       infoStreamPrint(LOG_STATS, 0, "%5d error test failures", solverInfo->solverStats[3]);
       infoStreamPrint(LOG_STATS, 0, "%5d convergence test failures", solverInfo->solverStats[4]);
       infoStreamPrint(LOG_STATS, 0, "%gs time of jacobian evaluation", rt_accumulated(SIM_TIMER_JACOBIAN));
+#ifdef USE_PARJAC
+      infoStreamPrint(LOG_STATS, 0, "%i OpenMP-threads used for jacobian evaluation", omc_get_max_threads());
+      int chunk_size;
+      omp_sched_t kind;
+      omp_get_schedule(&kind, &chunk_size);
+      infoStreamPrint(LOG_STATS, 0, "Schedule: %i Chunk Size: %i", kind, chunk_size);
+#endif
+
       messageClose(LOG_STATS);
     }
 
     infoStreamPrint(LOG_STATS_V, 1, "function calls");
-    if (omc_flag[FLAG_DAE_MODE])
+    if (compiledInDAEMode)
     {
-      infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionDAE", data->simulationInfo->callStatistics.functionEvalDAE);
+      infoStreamPrint(LOG_STATS_V, 1, "%5ld calls of functionDAE", rt_ncall(SIM_TIMER_DAE));
+      infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_DAE), rt_accumulated(SIM_TIMER_DAE)/total100);
+      messageClose(LOG_STATS_V);
     }
-    else
-    {
-      infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionODE", data->simulationInfo->callStatistics.functionODE);
+    if (data->simulationInfo->callStatistics.functionODE) {
+      infoStreamPrint(LOG_STATS_V, 1, "%5ld calls of functionODE", data->simulationInfo->callStatistics.functionODE);
+      infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_FUNCTION_ODE), rt_accumulated(SIM_TIMER_FUNCTION_ODE)/total100);
+      messageClose(LOG_STATS_V);
     }
-    infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionAlgebraics", data->simulationInfo->callStatistics.functionAlgebraics);
+
+    if (rt_ncall(SIM_TIMER_RESIDUALS)) {
+      infoStreamPrint(LOG_STATS_V, 1, "%5d calls of functionODE_residual", rt_ncall(SIM_TIMER_RESIDUALS));
+      infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_RESIDUALS), rt_accumulated(SIM_TIMER_RESIDUALS)/total100);
+      messageClose(LOG_STATS_V);
+    }
+
+    if (data->simulationInfo->callStatistics.functionAlgebraics) {
+      infoStreamPrint(LOG_STATS_V, 1, "%5d calls of functionAlgebraics", data->simulationInfo->callStatistics.functionAlgebraics);
+      infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_ALGEBRAICS), rt_accumulated(SIM_TIMER_ALGEBRAICS)/total100);
+      messageClose(LOG_STATS_V);
+    }
+
+    infoStreamPrint(LOG_STATS_V, 1, "%5d evaluations of jacobian", rt_ncall(SIM_TIMER_JACOBIAN));
+    infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_JACOBIAN), rt_accumulated(SIM_TIMER_JACOBIAN)/total100);
+    messageClose(LOG_STATS_V);
+
     infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of updateDiscreteSystem", data->simulationInfo->callStatistics.updateDiscreteSystem);
     infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionZeroCrossingsEquations", data->simulationInfo->callStatistics.functionZeroCrossingsEquations);
-    infoStreamPrint(LOG_STATS_V, 0, "%5ld calls of functionZeroCrossings", data->simulationInfo->callStatistics.functionZeroCrossings);
+
+    infoStreamPrint(LOG_STATS_V, 1, "%5ld calls of functionZeroCrossings", data->simulationInfo->callStatistics.functionZeroCrossings);
+    infoStreamPrint(LOG_STATS_V, 0, "%12gs [%5.1f%%]", rt_accumulated(SIM_TIMER_ZC), rt_accumulated(SIM_TIMER_ZC)/total100);
+    messageClose(LOG_STATS_V);
+
     messageClose(LOG_STATS_V);
 
     infoStreamPrint(LOG_STATS_V, 1, "linear systems");
@@ -666,10 +710,12 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
    */
   simInfo->useStopTime = 1;
 
-  /* if the given step size is too small redefine it */
-  if ((simInfo->stepSize < MINIMAL_STEP_SIZE) && (simInfo->stopTime > 0)){
-    warningStreamPrint(LOG_STDOUT, 0, "The step-size %g is too small. Adjust the step-size to %g.", simInfo->stepSize, MINIMAL_STEP_SIZE);
-    simInfo->stepSize = MINIMAL_STEP_SIZE;
+  /* Use minStepSize if stepSize is getting too small, but
+   * allow stepSize to be zero if startTime == stopTime.
+   */
+  if ((simInfo->stepSize < simInfo->minStepSize) && (simInfo->stopTime > simInfo->startTime)){
+    warningStreamPrint(LOG_STDOUT, 0, "The step-size %g is too small. Adjust the step-size to %g.", simInfo->stepSize, simInfo->minStepSize);
+    simInfo->stepSize = simInfo->minStepSize;
     simInfo->numSteps = round((simInfo->stopTime - simInfo->startTime)/simInfo->stepSize);
   }
 #if !defined(OMC_EMCC)
@@ -680,15 +726,19 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
   externalInputallocate(data);
   /* set tolerance for ZeroCrossings */
   setZCtol(fmin(data->simulationInfo->stepSize, data->simulationInfo->tolerance));
-
   omc_alloc_interface.collect_a_little();
+
+  /* initialize solver data */
+  /* For the DAEmode we need to initialize solverData before the initialization,
+   * since the solver is used to obtain consistent values also via updateDiscreteSystem
+   */
+  retVal = initializeSolverData(data, threadData, &solverInfo);
+  initSolverInfo = 1;
+
   /* initialize all parts of the model */
-  retVal = initializeModel(data, threadData, init_initMethod, init_file, init_time);
-  omc_alloc_interface.collect_a_little();
-
-  if(0 == retVal) {
-    retVal = initializeSolverData(data, threadData, &solverInfo);
-    initSolverInfo = 1;
+  if (0 == retVal){
+    retVal = initializeModel(data, threadData, init_initMethod, init_file, init_time);
+    omc_alloc_interface.collect_a_little();
   }
 
 #if !defined(OMC_MINIMAL_RUNTIME)
@@ -791,7 +841,7 @@ int solver_main(DATA* data, threadData_t *threadData, const char* init_initMetho
   }
 
   if (!retVal)
-    infoStreamPrint(LOG_STDOUT, 0, "The simulation finished successfully.");
+    infoStreamPrint(LOG_SUCCESS, 0, "The simulation finished successfully.");
 
   TRACE_POP
   return retVal;
@@ -805,6 +855,8 @@ static int euler_ex_step(DATA* data, SOLVER_INFO* solverInfo)
   SIMULATION_DATA *sDataOld = (SIMULATION_DATA*)data->localData[1];
   modelica_real* stateDer = sDataOld->realVars + data->modelData->nStates;
 
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   solverInfo->currentTime = sDataOld->timeValue + solverInfo->currentStepSize;
 
   for(i = 0; i < data->modelData->nStates; i++)
@@ -819,6 +871,8 @@ static int euler_ex_step(DATA* data, SOLVER_INFO* solverInfo)
   /* function ODE evaluation is done directly after this function */
   solverInfo->solverStatsTmp[1] += 1;
 
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
+
   return 0;
 }
 
@@ -1020,6 +1074,8 @@ static int rungekutta_step(DATA* data, threadData_t *threadData, SOLVER_INFO* so
   modelica_real* stateDer = sData->realVars + data->modelData->nStates;
   modelica_real* stateDerOld = sDataOld->realVars + data->modelData->nStates;
 
+  if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
+
   solverInfo->currentTime = sDataOld->timeValue + solverInfo->currentStepSize;
 
   /* We calculate k[0] before returning from this function.
@@ -1033,11 +1089,13 @@ static int rungekutta_step(DATA* data, threadData_t *threadData, SOLVER_INFO* so
       sData->realVars[i] = sDataOld->realVars[i] + solverInfo->currentStepSize * rk->c[j] * k[j - 1][i];
     }
     sData->timeValue = sDataOld->timeValue + rk->c[j] * solverInfo->currentStepSize;
+    if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
     /* read input vars */
     externalInputUpdate(data);
     data->callback->input_function(data, threadData);
     /* eval ode equations */
     data->callback->functionODE(data, threadData);
+    if (measure_time_flag) rt_tick(SIM_TIMER_SOLVER);
     memcpy(k[j], stateDer, data->modelData->nStates*sizeof(modelica_real));
 
   }
@@ -1058,6 +1116,7 @@ static int rungekutta_step(DATA* data, threadData_t *threadData, SOLVER_INFO* so
   solverInfo->solverStatsTmp[0] += 1;
   /* function ODE evaluation is done directly after this */
   solverInfo->solverStatsTmp[1] += rk->work_states_ndims+1;
+  if (measure_time_flag) rt_accumulate(SIM_TIMER_SOLVER);
 
   return 0;
 }
