@@ -4,34 +4,22 @@
  *  @{
  */
 
-
-/*includes removed for static linking not needed any more
-#ifdef RUNTIME_STATIC_LINKING
-#include <sstream>
-#include <vector>
-#endif
-
-// Output
+#include <Core/DataExchange/FactoryPolicy.h>
 #include <fstream>
-#include <Core/DataExchange/Writer.h>
-using std::ios;
-#define SEPERATOR ","
-#define EXTENSION ","
-*/
+
 /**
  Policy class to write simulation results in a text file
- */
-  const char SEPERATOR = ',';
-    const char EXTENSION = ',';
-template<size_t dim_1, size_t dim_2, size_t dim_3, size_t dim_4>
-class TextFileWriter : public Writer<dim_1, dim_2, dim_3, dim_4>
+*/
+const char SEPERATOR = ',';
+const char EXTENSION = ',';
+
+class TextFileWriter : public ContainerManager
 {
  public:
-    TextFileWriter(unsigned long size, string output_path, string file_name)
-            : Writer<dim_1, dim_2, dim_3, dim_4>(),
+    TextFileWriter(unsigned long size, string file_name)
+            : ContainerManager(),
               _output_stream(),
               _curser_position(0),
-              _output_path(output_path),
               _file_name(file_name)
     {
     }
@@ -42,16 +30,15 @@ class TextFileWriter : public Writer<dim_1, dim_2, dim_3, dim_4>
             _output_stream.close();
     }
 
-    void init(std::string output_path, std::string file_name)
+    void init(std::string file_name, size_t dim)
     {
         _file_name = file_name;
-        _output_path = output_path;
         if (_output_stream.is_open())
             _output_stream.close();
-        std::stringstream res_output_path;
-        res_output_path << output_path << file_name;
-        _output_stream.open(res_output_path.str().c_str(), ios::out);
 
+        _output_stream.open(file_name.c_str(), ios::out);
+        if (_output_stream.fail())
+          throw ModelicaSimulationError(DATASTORAGE, string("Failed to open results file ") + file_name);
     }
     void read(ublas::matrix<double>& R, ublas::matrix<double>& dR)
     {
@@ -82,7 +69,7 @@ class TextFileWriter : public Writer<dim_1, dim_2, dim_3, dim_4>
      @start_time
      @end_time
      */
-    void write(const typename Writer<dim_1, dim_2, dim_3, dim_4>::value_type_p& v_list, double start_time, double end_time)
+    virtual void write(const all_vars_t& v_list, const neg_all_vars_t& neg_v_list, double start_time, double end_time)
     {
 
         //not supported for file output
@@ -95,14 +82,17 @@ class TextFileWriter : public Writer<dim_1, dim_2, dim_3, dim_4>
      @s_parameter_list name of parameter
      @s_desc_parameter_list description of parameter
      */
-    void write(const std::vector<std::string>& s_list, const std::vector<std::string>& s_desc_list, const std::vector<std::string>& s_parameter_list, const std::vector<std::string>& s_desc_parameter_list)
+    virtual void write(const all_names_t& s_list,const all_description_t& s_desc_list, const all_names_t& s_parameter_list,const all_description_t& s_desc_parameter_list)
     {
         std::string s;
         _output_stream << "\"time\"" << SEPERATOR;
 
-        for (std::vector<std::string>::const_iterator it = s_list.begin(); it != s_list.end(); ++it)
+        for (var_names_t::const_iterator it = get<0>(s_list).begin(); it != get<0>(s_list).end(); ++it)
+            _output_stream << "\"" << (*it) << "\"" << SEPERATOR;
+        for (var_names_t::const_iterator it = get<1>(s_list).begin(); it != get<1>(s_list).end(); ++it)
+            _output_stream << "\"" << (*it) << "\"" << SEPERATOR;
+        for (var_names_t::const_iterator it = get<2>(s_list).begin(); it != get<2>(s_list).end(); ++it)
             _output_stream << "\"" << (*it) << "\"" << SEPERATOR;
-
         _output_stream << std::endl;
     }
 
@@ -117,15 +107,21 @@ class TextFileWriter : public Writer<dim_1, dim_2, dim_3, dim_4>
      @v2_list derivatives vars
      @time
      */
-    void write(typename Writer<dim_1, dim_2, dim_3, dim_4>::value_type_v& v_list, typename Writer<dim_1, dim_2, dim_3, dim_4>::value_type_dv& v2_list, double time)
+    virtual void write(const all_vars_time_t& v_list,const neg_all_vars_t& neg_v_list)
     {
-        _output_stream << time << SEPERATOR;
+        _output_stream << get<3>(v_list) << SEPERATOR;
 
-        for (typename Writer<dim_1, dim_2, dim_3, dim_4>::value_type_v::const_iterator it = v_list.begin(); it != v_list.end(); ++it)
-            _output_stream << (*it) << SEPERATOR;
 
-        for (typename Writer<dim_1, dim_2, dim_3, dim_4>::value_type_dv::const_iterator it = v2_list.begin(); it != v2_list.end(); ++it)
-            _output_stream << (*it) << SEPERATOR;
+        std::transform(get<0>(v_list).begin(), get<0>(v_list).end(), get<0>(neg_v_list).begin(),
+            std::ostream_iterator<double>(_output_stream,","), WriteOutputVar<double>());
+
+
+        std::transform(get<1>(v_list).begin(), get<1>(v_list).end(), get<1>(neg_v_list).begin(),
+            std::ostream_iterator<int>(_output_stream,","), WriteOutputVar<int>());
+
+
+        std::transform(get<2>(v_list).begin(), get<2>(v_list).end(), get<2>(neg_v_list).begin(),
+           std::ostream_iterator<bool>(_output_stream,","), WriteOutputVar<bool>());
 
         _output_stream << std::endl;
     }
@@ -152,8 +148,7 @@ class TextFileWriter : public Writer<dim_1, dim_2, dim_3, dim_4>
 
     std::fstream _output_stream;
     unsigned int _curser_position;       ///< Controls current Curser-Position
-    std::string _output_path;
     std::string _file_name;
     vector<string> _var_outputs;
 };
-/** @} */ // end of dataexchangePolicies
\ No newline at end of file
+/** @} */ // end of dataexchangePolicies
