@@ -21,14 +21,17 @@ public:
   ///< Output step size (default: 20 ms)
   virtual double gethOutput();
   virtual void sethOutput(double);
-  ///< Write out results ([false,true]; default: true)
-  virtual bool getResultsOutput();
-  virtual void setResultsOutput(bool);
+  ///< Write out results (default: EMIT_ALL)
+  virtual EmitResults getEmitResults();
+  virtual void setEmitResults(EmitResults);
   ///< Write out statistical simulation infos, e.g. number of steps (at the end of simulation); [false, true]; default: true)
   virtual bool getInfoOutput();
   virtual void setInfoOutput(bool);
   virtual bool useEndlessSim();
   virtual void useEndlessSim(bool);
+  ///path for input files, like init xml
+  virtual string getInputPath();
+  virtual void setInputPath(string);
   ///path for simulation results in textfile
   virtual string getOutputPath();
   virtual void setOutputPath(string);
@@ -36,7 +39,8 @@ public:
   virtual void setOutputPointType(OutputPointType);
   virtual LogSettings getLogSettings();
   virtual void setLogSettings(LogSettings);
-
+  virtual OutputFormat getOutputFormat();
+  virtual void setOutputFormat(OutputFormat);
   //solver used for simulation
   virtual string getSelectedSolver();
   virtual void setSelectedSolver(string);
@@ -53,20 +57,26 @@ public:
   virtual void setAlarmTime(unsigned int);
   virtual unsigned int getAlarmTime();
 
+
   virtual void setNonLinearSolverContinueOnError(bool);
   virtual bool getNonLinearSolverContinueOnError();
 
+  virtual void setSolverThreads(int);
+  virtual int getSolverThreads();
+
 private:
   double
-      _startTime, ///< Start time of integration (default: 0.0)
-      _endTime, ///< End time of integraiton (default: 1.0)
-      _hOutput; //< Output step size (default: 20 ms)
+      _startTime,   ///< Start time of integration (default: 0.0)
+      _endTime,     ///< End time of integraiton (default: 1.0)
+      _hOutput;     ///< Output step size (default: 20 ms)
+  EmitResults
+      _emitResults; ///< Write out results (default: EMIT_ALL)
   bool
-      _resultsOutput,   ///< Write out results ([false,true]; default: true)
-      _infoOutput,      ///< Write out statistical simulation infos, e.g. number of steps (at the end of simulation); [false,true]; default: true)
+      _infoOutput,  ///< Write out statistical simulation infos, e.g. number of steps (at the end of simulation); [false,true]; default: true)
       _endless_sim,
       _nonLinSolverContinueOnError;
   string
+      _input_path,
       _output_path,
       _selected_solver,
       _selected_lin_solver,
@@ -76,5 +86,8 @@ private:
   OutputPointType _outputPointType;
   LogSettings _log_settings;
   unsigned int _alarm_time;
+
+  int _solverThreads;
+  OutputFormat _outputFormat;
 };
 /** @} */ // end of coreSimulationSettings
