@@ -74,7 +74,8 @@ AbstractAnimationWindow::AbstractAnimationWindow(QWidget *pParent)
     mpPerspectiveDropDownBox(nullptr),
     mpRotateCameraLeftAction(nullptr),
     mpRotateCameraRightAction(nullptr),
-    mCameraInitialized(false)
+    mCameraInitialized(false),
+    mSliderRange(1000)
 {
   // to distinguish this widget as a subwindow among the plotwindows
   setObjectName(QString("animationWindow"));
@@ -171,7 +172,7 @@ void AbstractAnimationWindow::createActions()
   // animation slide
   mpAnimationSlider = new QSlider(Qt::Horizontal);
   mpAnimationSlider->setMinimum(0);
-  mpAnimationSlider->setMaximum(100);
+  mpAnimationSlider->setMaximum(mSliderRange);
   mpAnimationSlider->setSliderPosition(0);
   mpAnimationSlider->setEnabled(false);
   connect(mpAnimationSlider, SIGNAL(valueChanged(int)),this, SLOT(sliderSetTimeSlotFunction(int)));
@@ -303,17 +304,15 @@ void AbstractAnimationWindow::initInteractiveControlPanel()
     QScrollArea *scrollArea = new QScrollArea(this);
     scrollArea->setWidget(widget);
     mpAnimationParameterDockerWidget->setWidget(scrollArea);
-    }
-    else {
-        MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, tr("Information about states could not be determined."),
-                                                                Helper::scriptingKind, Helper::errorLevel));
+    } else {
+      MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, tr("Information about states could not be determined."),
+                                                            Helper::scriptingKind, Helper::errorLevel));
         mpAnimationParameterDockerWidget->hide();
 
     }
-  }
-  else {
-    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, tr("Interactive Control needs an FMU ME 2.0"),
-                                                            Helper::scriptingKind, Helper::errorLevel));
+  } else {
+    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, tr("Interactive Control needs an FMU ME 2.0"),
+                                                          Helper::scriptingKind, Helper::errorLevel));
     mpAnimationParameterDockerWidget->hide();
   }
 }
@@ -377,7 +376,7 @@ bool AbstractAnimationWindow::loadVisualization()
   } else if (isCSV(mFileName)) {
     visType = VisType::CSV;
   } else {
-    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, tr("Unknown visualization type."),
+    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, tr("Unknown visualization type."),
                                                           Helper::scriptingKind, Helper::errorLevel));
     return false;
   }
@@ -385,7 +384,7 @@ bool AbstractAnimationWindow::loadVisualization()
   bool xmlExists = checkForXMLFile(mFileName, mPathName);
   if (!xmlExists) {
     QString msg = tr("Could not find the visual XML file %1.").arg(QString(assembleXMLFileName(mFileName, mPathName).c_str()));
-    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, msg, Helper::scriptingKind,
+    MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, msg, Helper::scriptingKind,
                                                           Helper::errorLevel));
     return false;
   } else {
@@ -398,7 +397,7 @@ bool AbstractAnimationWindow::loadVisualization()
       mpVisualizer = new VisualizerFMU(mFileName, mPathName);
     } else {
       QString msg = tr("Could not init %1 %2.").arg(QString(mPathName.c_str())).arg(QString(mFileName.c_str()));
-      MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, "", false, 0, 0, 0, 0, msg, Helper::scriptingKind,
+      MessagesWidget::instance()->addGUIMessage(MessageItem(MessageItem::Modelica, msg, Helper::scriptingKind,
                                                             Helper::errorLevel));
       return false;
     }
@@ -611,7 +610,7 @@ void AbstractAnimationWindow::sliderSetTimeSlotFunction(int value)
 {
   float time = (mpVisualizer->getTimeManager()->getEndTime()
                 - mpVisualizer->getTimeManager()->getStartTime())
-      * (float) (value / 100.0);
+      * (float) (value / (float)mSliderRange);
   mpVisualizer->getTimeManager()->setVisTime(time);
   mpTimeTextBox->setText(QString::number(mpVisualizer->getTimeManager()->getVisTime()));
   mpVisualizer->updateScene(time);
