@@ -46,7 +46,7 @@
  * \param pGraphicsView - pointer to GraphicsView
  */
 TextAnnotation::TextAnnotation(QString annotation, GraphicsView *pGraphicsView)
-  : ShapeAnnotation(false, pGraphicsView, 0)
+  : ShapeAnnotation(false, pGraphicsView, 0, 0)
 {
   mpComponent = 0;
   // set the default values
@@ -60,42 +60,33 @@ TextAnnotation::TextAnnotation(QString annotation, GraphicsView *pGraphicsView)
 }
 
 TextAnnotation::TextAnnotation(ShapeAnnotation *pShapeAnnotation, Component *pParent)
-  : ShapeAnnotation(pParent), mpComponent(pParent)
+  : ShapeAnnotation(pShapeAnnotation, pParent), mpComponent(pParent)
 {
   updateShape(pShapeAnnotation);
-  initUpdateVisible(); // DynamicSelect for visible attribute
   initUpdateTextString();
   setPos(mOrigin);
   setRotation(mRotation);
-  connect(pShapeAnnotation, SIGNAL(updateReferenceShapes()), pShapeAnnotation, SIGNAL(changed()));
-  connect(pShapeAnnotation, SIGNAL(added()), this, SLOT(referenceShapeAdded()));
-  connect(pShapeAnnotation, SIGNAL(changed()), this, SLOT(referenceShapeChanged()));
-  connect(pShapeAnnotation, SIGNAL(deleted()), this, SLOT(referenceShapeDeleted()));
 }
 
 TextAnnotation::TextAnnotation(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView)
-  : ShapeAnnotation(true, pGraphicsView, 0)
+  : ShapeAnnotation(true, pGraphicsView, pShapeAnnotation, 0)
 {
   mpComponent = 0;
   updateShape(pShapeAnnotation);
   setShapeFlags(true);
   mpGraphicsView->addItem(this);
-  connect(pShapeAnnotation, SIGNAL(updateReferenceShapes()), pShapeAnnotation, SIGNAL(changed()));
-  connect(pShapeAnnotation, SIGNAL(added()), this, SLOT(referenceShapeAdded()));
-  connect(pShapeAnnotation, SIGNAL(changed()), this, SLOT(referenceShapeChanged()));
-  connect(pShapeAnnotation, SIGNAL(deleted()), this, SLOT(referenceShapeDeleted()));
 }
 
 TextAnnotation::TextAnnotation(Component *pParent)
-  : ShapeAnnotation(pParent), mpComponent(pParent)
+  : ShapeAnnotation(0, pParent), mpComponent(pParent)
 {
   // set the default values
   GraphicItem::setDefaults();
   FilledShape::setDefaults();
   ShapeAnnotation::setDefaults();
   // give a reasonable size to default component text
-  mExtents.replace(0, QPointF(-50, -50));
-  mExtents.replace(1, QPointF(50, 50));
+  mExtents.replace(0, QPointF(-100, -50));
+  mExtents.replace(1, QPointF(100, 50));
   setTextString("%name");
   initUpdateTextString();
   setPos(mOrigin);
@@ -103,7 +94,7 @@ TextAnnotation::TextAnnotation(Component *pParent)
 }
 
 TextAnnotation::TextAnnotation(QString annotation, LineAnnotation *pLineAnnotation)
-  : ShapeAnnotation(pLineAnnotation)
+  : ShapeAnnotation(0, pLineAnnotation)
 {
   mpComponent = 0;
   // set the default values
@@ -132,7 +123,7 @@ TextAnnotation::TextAnnotation(QString annotation, LineAnnotation *pLineAnnotati
  * \param pGraphicsView
  */
 TextAnnotation::TextAnnotation(GraphicsView *pGraphicsView)
-  : ShapeAnnotation(true, pGraphicsView, 0)
+  : ShapeAnnotation(true, pGraphicsView, 0, 0)
 {
   mpComponent = 0;
   // set the default values
@@ -160,7 +151,7 @@ void TextAnnotation::parseShapeAnnotation(QString annotation)
   FilledShape::parseShapeAnnotation(annotation);
   // parse the shape to get the list of attributes of Text.
   QStringList list = StringHandler::getStrings(annotation);
-  if (list.size() < 11) {
+  if (list.size() < 15) {
     return;
   }
   // 9th item of the list contains the extent points
@@ -174,54 +165,58 @@ void TextAnnotation::parseShapeAnnotation(QString annotation)
   if (list.at(9).startsWith("{")) {
     // DynamicSelect
     QStringList args = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(9)));
-    if (args.count() > 0)
+    if (args.count() > 0) {
       mOriginalTextString = StringHandler::removeFirstLastQuotes(args.at(0));
-    if (args.count() > 1)
+    }
+    if (args.count() > 1) {
       mDynamicTextString << args.at(1);  // variable name
-    if (args.count() > 2)
+    }
+    if (args.count() > 2) {
       mDynamicTextString << args.at(2);  // significantDigits
-  }
-  else {
+    }
+  } else {
     mOriginalTextString = StringHandler::removeFirstLastQuotes(list.at(9));
   }
   mTextString = mOriginalTextString;
   initUpdateTextString();
   // 11th item of the list contains the fontSize.
   mFontSize = list.at(10).toFloat();
-  //Now comes the optional parameters; fontName and textStyle.
-  annotation = annotation.replace("{", "");
-  annotation = annotation.replace("}", "");
-  // parse the shape to get the list of attributes of Text Annotation.
-  list = StringHandler::getStrings(annotation);
-  int index = 19;
-  mTextStyles.clear();
-  while(index < list.size()) {
-    QString annotationValue = StringHandler::removeFirstLastQuotes(list.at(index));
-    // check textStyles enumeration.
-    if(annotationValue == "TextStyle.Bold") {
-      mTextStyles.append(StringHandler::TextStyleBold);
-      index++;
-    } else if(annotationValue == "TextStyle.Italic") {
-      mTextStyles.append(StringHandler::TextStyleItalic);
-      index++;
-    } else if(annotationValue == "TextStyle.UnderLine") {
-      mTextStyles.append(StringHandler::TextStyleUnderLine);
-      index++;
-    } else if(annotationValue == "TextAlignment.Left") {
-      // check textAlignment enumeration.
-      mHorizontalAlignment = StringHandler::TextAlignmentLeft;
-      index++;
-    } else if(annotationValue == "TextAlignment.Center") {
-      mHorizontalAlignment = StringHandler::TextAlignmentCenter;
-      index++;
-    } else if(annotationValue == "TextAlignment.Right") {
-      mHorizontalAlignment = StringHandler::TextAlignmentRight;
-      index++;
-    } else {
-      mFontName = annotationValue;
-      index++;
+  // 12th item of the list contains the optional textColor, {-1, -1, -1} if not set
+  QStringList textColorList = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(11)));
+  if (textColorList.size() >= 3) {
+    int red, green, blue = 0;
+    red = textColorList.at(0).toInt();
+    green = textColorList.at(1).toInt();
+    blue = textColorList.at(2).toInt();
+    if (red >= 0 && green >= 0 && blue >= 0) {
+      mLineColor = QColor (red, green, blue);
     }
   }
+  // 13th item of the list contains the font name.
+  QString fontName = StringHandler::removeFirstLastQuotes(list.at(12));
+  if (!fontName.isEmpty()) {
+    mFontName = fontName;
+  }
+  // 14th item of the list contains the text styles.
+  QStringList textStyles = StringHandler::getStrings(StringHandler::removeFirstLastCurlBrackets(list.at(13)));
+  foreach (QString textStyle, textStyles) {
+    if (textStyle == "TextStyle.Bold") {
+      mTextStyles.append(StringHandler::TextStyleBold);
+    } else if (textStyle == "TextStyle.Italic") {
+      mTextStyles.append(StringHandler::TextStyleItalic);
+    } else if (textStyle == "TextStyle.UnderLine") {
+      mTextStyles.append(StringHandler::TextStyleUnderLine);
+    }
+  }
+  // 15th item of the list contains the text alignment.
+  QString horizontalAlignment = StringHandler::removeFirstLastQuotes(list.at(14));
+  if (horizontalAlignment == "TextAlignment.Left") {
+    mHorizontalAlignment = StringHandler::TextAlignmentLeft;
+  } else if (horizontalAlignment == "TextAlignment.Center") {
+    mHorizontalAlignment = StringHandler::TextAlignmentCenter;
+  } else if (horizontalAlignment == "TextAlignment.Right") {
+    mHorizontalAlignment = StringHandler::TextAlignmentRight;
+  }
 }
 
 /*!
@@ -257,12 +252,9 @@ void TextAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
 {
   Q_UNUSED(option);
   Q_UNUSED(widget);
-  //! @note We don't show text annotation that contains % for Library Icons. Only static text for functions are shown.
+  //! @note We don't show text annotation that contains % for Library Icons or if it is too long.
   if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap()) {
-    if (mpGraphicsView->getModelWidget()->getLibraryTreeItem()->getRestriction() != StringHandler::Function) {
-      return;
-    }
-    if (mOriginalTextString.contains("%")) {
+    if (mOriginalTextString.contains("%") || mOriginalTextString.length() > maxTextLengthToShowOnLibraryIcon) {
       return;
     }
   } else if (mpComponent && mpComponent->getGraphicsView()->isRenderingLibraryPixmap()) {
@@ -289,6 +281,10 @@ void TextAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
         painter->setOpacity(0.2);
       }
     }
+    if (!mDynamicVisibleValue && ((mpGraphicsView && mpGraphicsView->isVisualizationView())
+                                  || (mpParentComponent && mpParentComponent->getGraphicsView()->isVisualizationView()))) {
+      return;
+    }
     drawTextAnnotaion(painter);
   }
 }
@@ -303,96 +299,88 @@ void TextAnnotation::drawTextAnnotaion(QPainter *painter)
   applyLinePattern(painter);
   /* Don't apply the fill patterns on Text shapes. */
   /*applyFillPattern(painter);*/
-  qreal dx = ((-boundingRect().left()) - boundingRect().right());
-  qreal dy = ((-boundingRect().top()) - boundingRect().bottom());
-  // first we invert the painter since we have our coordinate system inverted.
-  painter->scale(1.0, -1.0);
-  painter->translate(0, dy);
+  // store the existing transformations
+  const QTransform painterTransform = painter->transform();
+  const qreal scaleX = painterTransform.m11();
+  const qreal scaleY = painterTransform.m22();
+  const qreal shearX = painterTransform.m21();
+  const qreal shearY = painterTransform.m12();
+  // set new transformation for the text based on rotation or scale.
+  qreal sx, sy;
+  if (painterTransform.type() == QTransform::TxRotate) {
+    // if no flip or XY flip
+    if ((shearX >= 0 && shearY >= 0) || (shearX < 0 && shearY < 0)) {
+      painter->setTransform(QTransform(painterTransform.m11(), (shearY < 0 ? -1.0 : 1.0), painterTransform.m13(),
+                                       (shearX >= 0 ? -1.0 : 1.0), painterTransform.m22(), painterTransform.m23(),
+                                       painterTransform.m31(), painterTransform.m32(), painterTransform.m33()));
+      sx = shearX * (shearX < 0 ? -1.0 : 1.0);
+      sy = shearY * (shearY >= 0 ? -1.0 : 1.0);
+    } else { // if x or y flip
+      painter->setTransform(QTransform(painterTransform.m11(), (shearY < 0 ? 1.0 : -1.0), painterTransform.m13(),
+                                       (shearX >= 0 ? -1.0 : 1.0), painterTransform.m22(), painterTransform.m23(),
+                                       painterTransform.m31(), painterTransform.m32(), painterTransform.m33()));
+      sx = shearX * (shearX < 0 ? 1.0 : -1.0);
+      sy = shearY * (shearY >= 0 ? -1.0 : 1.0);
+    }
+  } else {
+    painter->setTransform(QTransform(1.0, painterTransform.m12(), painterTransform.m13(),
+                                     painterTransform.m21(), 1.0, painterTransform.m23(),
+                                     painterTransform.m31(), painterTransform.m32(), painterTransform.m33()));
+    sx = scaleX;
+    sy = scaleY;
+  }
+  // map the existing bounding rect to new transformation
+  QRectF mappedBoundingRect = QRectF(boundingRect().x() * sx, boundingRect().y() * sy, boundingRect().width() * sx, boundingRect().height() * sy);
+  // map the existing bounding rect to new transformation but with positive width and height so that font metrics can work
+  QRectF absMappedBoundingRect = QRectF(boundingRect().x() * sx, boundingRect().y() * sy, qAbs(boundingRect().width() * sx), qAbs(boundingRect().height() * sy));
+  // normalize the text for drawing
   mTextString = StringHandler::removeFirstLastQuotes(mTextString);
   mTextString = StringHandler::unparse(QString("\"").append(mTextString).append("\""));
-  QFont font;
-  font = QFont(mFontName, mFontSize, StringHandler::getFontWeight(mTextStyles), StringHandler::getFontItalic(mTextStyles));
+  // Don't create new QFont instead get a font from painter and set the values on it and set it back.
+  QFont font = painter->font();
+  font.setFamily(mFontName);
+  if (mFontSize > 0) {
+    font.setPointSizeF(mFontSize);
+  }
+  font.setWeight(StringHandler::getFontWeight(mTextStyles));
+  font.setItalic(StringHandler::getFontItalic(mTextStyles));
   // set font underline
   if (StringHandler::getFontUnderline(mTextStyles)) {
     font.setUnderline(true);
   }
   painter->setFont(font);
-  QRect fontBoundRect = painter->fontMetrics().boundingRect(boundingRect().toRect(), Qt::TextDontClip, mTextString);
-  bool calculateFontSize = true;
-  if (mFontSize > 0) {
-    if (boundingRect().width() != 0 && fontBoundRect.width() <= boundingRect().width()) {
-      calculateFontSize = false;
-    } else if (boundingRect().height() != 0 && fontBoundRect.height() <= boundingRect().height()) {
-      calculateFontSize = false;
-    }
-  }
-  if (calculateFontSize) {
-    float xFactor = boundingRect().width() / fontBoundRect.width();
-    float yFactor = boundingRect().height() / fontBoundRect.height();
+  /* From Modelica specification version 3.5-dev
+   * "The style attribute fontSize specifies the font size. If the fontSize attribute is 0 the text is scaled to fit its extent. Otherwise, the size specifies the absolute size."
+   */
+  // if absolute font size is defined and is greater than 0 then we don't need to calculate the font size.
+  if (mFontSize <= 0) {
+    QFontMetrics fontMetrics(painter->font());
+    QRect fontBoundRect = fontMetrics.boundingRect(absMappedBoundingRect.toRect(), Qt::TextDontClip, mTextString);
+    const qreal xFactor = absMappedBoundingRect.width() / fontBoundRect.width();
+    const qreal yFactor = absMappedBoundingRect.height() / fontBoundRect.height();
     /* Ticket:4256
-       * Text aspect when x1=x2 i.e, width is 0.
-       * Use height.
-       */
-    float factor = (boundingRect().width() != 0 && xFactor < yFactor) ? xFactor : yFactor;
-    QFont f = painter->font();
-    qreal fontSizeFactor = f.pointSizeF()*factor;
-    if ((fontSizeFactor < 12) && mpComponent) {
-      f.setPointSizeF(12);
-    } else if (fontSizeFactor <= 0) {
-      f.setPointSizeF(1);
-    } else {
-      f.setPointSizeF(fontSizeFactor);
-    }
-    painter->setFont(f);
+     * Text aspect when x1=x2 i.e, width is 0.
+     * Use height.
+     */
+    const qreal factor = (absMappedBoundingRect.width() != 0 && xFactor < yFactor) ? xFactor : yFactor;
+    qreal fontSizeFactor = font.pointSizeF() * factor;
+    // Yes we don't go below Helper::minimumTextFontSize font pt.
+    font.setPointSizeF(qMax(fontSizeFactor, Helper::minimumTextFontSize));
+    painter->setFont(font);
   }
-  if (mpComponent) {
-    Component *pComponent = mpComponent->getRootParentComponent();
-    if (pComponent && pComponent->mTransformation.isValid()) {
-      QPointF extent1 = pComponent->mTransformation.getExtent1();
-      QPointF extent2 = pComponent->mTransformation.getExtent2();
-      qreal componentAngle = StringHandler::getNormalizedAngle(pComponent->mTransformation.getRotateAngle());
-      qreal shapeAngle = StringHandler::getNormalizedAngle(getRotation());
-      // if shape has its own angle
-      if (shapeAngle > 0) {
-        shapeAngle = StringHandler::getNormalizedAngle(pComponent->mTransformation.getRotateAngle() + getRotation());
-        if (shapeAngle == 180) {
-          painter->scale(-1.0, -1.0);
-          painter->translate(dx, dy);
-        }
-        if (extent2.x() < extent1.x()) {  // if vertical flip
-          painter->scale(1.0, -1.0);
-          painter->translate(0, dy);
-        }
-        if (extent2.y() < extent1.y()) {  // if horizontal flip
-          painter->scale(-1.0, 1.0);
-          painter->translate(dx, 0);
-        }
-      } else {
-        if (componentAngle == 180) {
-          painter->scale(-1.0, -1.0);
-          painter->translate(dx, dy);
-        }
-        if (extent2.x() < extent1.x()) {  // if horizontal flip
-          painter->scale(-1.0, 1.0);
-          painter->translate(dx, 0);
-        }
-        if (extent2.y() < extent1.y()) {  // if vertical flip
-          painter->scale(1.0, -1.0);
-          painter->translate(0, dy);
-        }
-      }
-    }
-  } else {
-    qreal angle = StringHandler::getNormalizedAngle(mTransformation.getRotateAngle());
-    if (angle == 180) {
-      painter->scale(-1.0, -1.0);
-      painter->translate(((-boundingRect().left()) - boundingRect().right()), ((-boundingRect().top()) - boundingRect().bottom()));
-    }
+  /* Get the elided text
+   * Try to get the elided text if the shape is inside a Component
+   * If the shape is not part of Component then only try to get the elided text if its font size <= Helper::minimumTextFontSize
+   */
+  QString textToDraw = mTextString;
+  if (absMappedBoundingRect.width() > 1 && painter->font().pointSizeF() <= Helper::minimumTextFontSize) {
+    QFontMetrics fontMetrics(painter->font());
+    textToDraw = fontMetrics.elidedText(mTextString, Qt::ElideRight, absMappedBoundingRect.width());
   }
   // draw the font
-  if (mpComponent || boundingRect().width() > 0 || boundingRect().height() > 0) {
-    painter->drawText(boundingRect(), StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, mTextString);
-    mExportBoundingRect = painter->boundingRect(boundingRect(), StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, mTextString);
+  if (mpComponent || mappedBoundingRect.width() != 0 || mappedBoundingRect.height() != 0) {
+    painter->drawText(mappedBoundingRect, StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, textToDraw);
+    mExportBoundingRect = painter->boundingRect(mappedBoundingRect, StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, textToDraw);
     if (mpComponent) {
       mExportBoundingRect = sceneTransform().mapRect(mExportBoundingRect);
     }
@@ -421,29 +409,44 @@ QString TextAnnotation::getOMCShapeAnnotation()
   annotationString.append(QString("\"").append(mOriginalTextString).append("\""));
   // get the font size
   annotationString.append(QString::number(mFontSize));
+  // get the text color
+  QString textColorString;
+  textColorString.append("{");
+  textColorString.append(QString::number(mLineColor.red())).append(",");
+  textColorString.append(QString::number(mLineColor.green())).append(",");
+  textColorString.append(QString::number(mLineColor.blue()));
+  textColorString.append("}");
+  annotationString.append(textColorString);
   // get the font name
-  if (!mFontName.isEmpty()) {
+  if (!mFontName.isEmpty() && mFontName.compare(Helper::systemFontInfo.family()) != 0) {
     annotationString.append(QString("\"").append(mFontName).append("\""));
+  } else {
+    annotationString.append(QString("\"\""));
   }
   // get the font styles
   QString textStylesString;
   QStringList stylesList;
-  if (mTextStyles.size() > 0) {
-    textStylesString.append("{");
-  }
+  textStylesString.append("{");
   for (int i = 0 ; i < mTextStyles.size() ; i++) {
     stylesList.append(StringHandler::getTextStyleString(mTextStyles[i]));
   }
-  if (mTextStyles.size() > 0) {
-    textStylesString.append(stylesList.join(","));
-    textStylesString.append("}");
-    annotationString.append(textStylesString);
-  }
+  textStylesString.append(stylesList.join(","));
+  textStylesString.append("}");
+  annotationString.append(textStylesString);
   // get the font horizontal alignment
   annotationString.append(StringHandler::getTextAlignmentString(mHorizontalAlignment));
   return annotationString.join(",");
 }
 
+/*!
+ * \brief TextAnnotation::getOMCShapeAnnotationWithShapeName
+ * \return the shape annotation in format as returned by OMC wrapped in Text keyword.
+ */
+QString TextAnnotation::getOMCShapeAnnotationWithShapeName()
+{
+  return QString("Text(%1)").arg(getOMCShapeAnnotation());
+}
+
 /*!
  * \brief TextAnnotation::getShapeAnnotation
  * \return the shape annotation in Modelica syntax.
@@ -509,7 +512,7 @@ void TextAnnotation::updateShape(ShapeAnnotation *pShapeAnnotation)
 void TextAnnotation::initUpdateTextString()
 {
   if (mpComponent) {
-    if (mOriginalTextString.contains("%") || mDynamicTextString.count() > 0) {
+    if (mOriginalTextString.contains("%")) {
       updateTextString();
       connect(mpComponent, SIGNAL(displayTextChanged()), SLOT(updateTextString()), Qt::UniqueConnection);
     }
@@ -525,7 +528,7 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
 {
   int pos = 0;
   while ((pos = regExp.indexIn(mTextString, pos)) != -1) {
-    QString variable = regExp.cap(0);
+    QString variable = regExp.cap(0).trimmed();
     if ((!variable.isEmpty()) && (variable.compare("%%") != 0) && (variable.compare("%name") != 0) && (variable.compare("%class") != 0)) {
       variable.remove("%");
       if (!variable.isEmpty()) {
@@ -533,21 +536,35 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
         /* Ticket:4204
          * If we have extend component then call Component::getParameterDisplayString from root component.
          */
-        if (mpComponent->getComponentType() == Component::Extend) {
-          textValue = mpComponent->getRootParentComponent()->getParameterDisplayString(variable);
-        } else {
-          textValue = mpComponent->getRootParentComponent()->getParameterDisplayString(variable);
-        }
+        textValue = mpComponent->getRootParentComponent()->getParameterDisplayString(variable);
         if (!textValue.isEmpty()) {
-          mTextString.replace(pos, regExp.matchedLength(), textValue);
+          QString unit = "";
+          QString displaytUnit = "";
+          Component *pComponent = mpComponent->getRootParentComponent()->getComponentByName(variable);
+          if (pComponent) {
+            displaytUnit = pComponent->getDerivedClassModifierValue("displaytUnit");
+            if (displaytUnit.isEmpty()) {
+              unit = pComponent->getDerivedClassModifierValue("unit");
+              displaytUnit = unit;
+            }
+          }
+          if (displaytUnit.isEmpty()) {
+            mTextString.replace(pos, regExp.matchedLength(), textValue);
+            pos += textValue.length();
+          } else {
+            QString textValueWithDisplayUnit = QString("%1 %2").arg(textValue, displaytUnit);
+            mTextString.replace(pos, regExp.matchedLength(), textValueWithDisplayUnit);
+            pos += textValueWithDisplayUnit.length();
+          }
         } else { /* if the value of %\\W* is empty then remove the % sign. */
           mTextString.replace(pos, 1, "");
         }
       } else { /* if there is just alone % then remove it. Because if you want to print % then use %%. */
         mTextString.replace(pos, 1, "");
       }
+    } else if (variable.compare("%%") == 0) { /* if string is %% then just move over it. We replace it with % in TextAnnotation::updateTextString(). */
+      pos += regExp.matchedLength();
     }
-    pos += regExp.matchedLength();
   }
 }
 
@@ -557,24 +574,6 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
  */
 void TextAnnotation::updateTextString()
 {
-  /* optional DynamicSelect of textString attribute */
-  QVariant dynamicValue; // isNull() per default
-  if (mDynamicTextString.count() > 0) {
-    dynamicValue = getDynamicValue(mDynamicTextString.at(0).toString());
-  }
-  if (!dynamicValue.isNull()) {
-    mTextString = dynamicValue.toString();
-    if (mTextString.isEmpty()) {
-      /* use variable name as default value if result not found */
-      mTextString = mDynamicTextString.at(0).toString();
-    }
-    else if (mDynamicTextString.count() > 1) {
-      int digits = mDynamicTextString.at(1).toInt();
-      mTextString = QString::number(mTextString.toDouble(), 'g', digits);
-    }
-    return;
-  }
-  /* alternatively use model provided value */
   /* From Modelica Spec 32revision2,
    * There are a number of common macros that can be used in the text, and they should be replaced when displaying
    * the text as follows:
