@@ -51,6 +51,7 @@ extern "C" {
 #include <QMainWindow>
 #include <QDialog>
 #include <QProgressBar>
+#include <QMimeData>
 #include <QDomDocument>
 #include <QStackedWidget>
 #include <QActionGroup>
@@ -138,7 +139,9 @@ public:
   void showProgressBar() {mpProgressBar->setVisible(true);}
   void hideProgressBar() {mpProgressBar->setVisible(false);}
   Label* getPositionLabel() {return mpPositionLabel;}
-  QTabBar* getPerspectiveTabBar() {return mpPerspectiveTabbar;}
+  bool isModelingPerspectiveActive();
+  bool isPlottingPerspectiveActive();
+  bool isDebuggingPerspectiveActive();
   QTimer* getAutoSaveTimer() {return mpAutoSaveTimer;}
   QAction* getSaveAction() {return mpSaveAction;}
   QAction* getSaveAsAction() {return mpSaveAsAction;}
@@ -165,6 +168,7 @@ public:
   QAction* getCheckAllModelsAction() {return mpCheckAllModelsAction;}
   QAction* getExportFMUAction() {return mpExportFMUAction;}
   QAction* getExportEncryptedPackageAction() {return mpExportEncryptedPackageAction;}
+  QAction* getExportRealonlyPackageAction() {return mpExportReadonlyPackageAction;}
   QAction* getExportXMLAction() {return mpExportXMLAction;}
   QAction* getExportFigaroAction() {return mpExportFigaroAction;}
   QAction* getLineShapeAction() {return mpLineShapeAction;}
@@ -201,12 +205,17 @@ public:
   QAction* getCommitFilesAction() {return mpCommitFilesAction;}
   QAction* getRevertCommitAction() {return mpRevertCommitAction;}
   QAction* getCleanWorkingDirectoryAction() {return mpCleanWorkingDirectoryAction;}
+  QToolBar* getShapesToolBar() const {return mpShapesToolBar;}
+  QToolBar* getCheckToolBar() const {return mpCheckToolBar;}
+  QToolBar* getSimulationToolBar() const {return mpSimulationToolBar;}
+  QToolBar* getTLMSimulationToolbar() const {return mpTLMSimulationToolbar;}
+  QToolBar* getOMSimulatorToobar() const {return mpOMSimulatorToobar;}
   void addRecentFile(const QString &fileName, const QString &encoding);
   void updateRecentFileActions();
-  void closeEvent(QCloseEvent *event);
+  void closeEvent(QCloseEvent *event) override;
   int askForExit();
   void beforeClosingMainWindow();
-  void openDroppedFile(QDropEvent *event);
+  void openDroppedFile(const QMimeData *pMimeData);
   void openResultFiles(QStringList fileNames);
   void simulate(LibraryTreeItem *pLibraryTreeItem);
   void simulateWithTransformationalDebugger(LibraryTreeItem *pLibraryTreeItem);
@@ -222,6 +231,7 @@ public:
   void checkAllModels(LibraryTreeItem *pLibraryTreeItem);
   void exportModelFMU(LibraryTreeItem *pLibraryTreeItem);
   void exportEncryptedPackage(LibraryTreeItem *pLibraryTreeItem);
+  void exportReadonlyPackage(LibraryTreeItem *pLibraryTreeItem);
   void exportModelXML(LibraryTreeItem *pLibraryTreeItem);
   void exportModelFigaro(LibraryTreeItem *pLibraryTreeItem);
   void fetchInterfaceData(LibraryTreeItem *pLibraryTreeItem, QString singleModel=QString());
@@ -307,6 +317,18 @@ private:
   QAction *mpSaveAsAction;
   QAction *mpSaveAllAction;
   QAction *mpSaveTotalAction;
+  QAction *mpImportFMUAction;
+  QAction *mpImportFMUModelDescriptionAction;
+  QAction *mpImportFromOMNotebookAction;
+  QAction *mpImportNgspiceNetlistAction;
+  QAction *mpExportToClipboardAction;
+  QAction *mpExportAsImageAction;
+  QAction *mpExportFMUAction;
+  QAction *mpExportReadonlyPackageAction;
+  QAction *mpExportEncryptedPackageAction;
+  QAction *mpExportXMLAction;
+  QAction *mpExportFigaroAction;
+  QAction *mpExportToOMNotebookAction;
   QAction *mpRecentFileActions[MaxRecentFiles];
   QAction *mpClearRecentFilesAction;
   QAction *mpPrintModelAction;
@@ -315,9 +337,6 @@ private:
   QAction *mpUndoAction;
   QAction *mpRedoAction;
   QAction *mpFilterClassesAction;
-  QAction *mpCutAction;
-  QAction *mpCopyAction;
-  QAction *mpPasteAction;
   // View Menu
   QAction *mpShowGridLinesAction;
   QAction *mpResetZoomAction;
@@ -341,14 +360,6 @@ private:
   QAction *mpSimulateWithAnimationAction;
 #endif
   QAction *mpSimulationSetupAction;
-  // FMI Menu
-  QAction *mpExportFMUAction;
-  QAction *mpImportFMUAction;
-  QAction *mpImportFMUModelDescriptionAction;
-  // Export Menu
-  QAction *mpExportEncryptedPackageAction;
-  QAction *mpExportXMLAction;
-  QAction *mpExportFigaroAction;
   // Debug Menu
   QAction *mpDebugConfigurationsAction;
   QAction *mpAttachDebuggerToRunningProcessAction;
@@ -364,9 +375,6 @@ private:
   QAction *mpShowOMCLoggerWidgetAction;
   QAction *mpShowOpenModelicaCommandPromptAction;
   QAction *mpShowOMCDiffWidgetAction;
-  QAction *mpExportToOMNotebookAction;
-  QAction *mpImportFromOMNotebookAction;
-  QAction *mpImportNgspiceNetlistAction;
   QAction *mpOpenWorkingDirectoryAction;
   QAction *mpOpenTerminalAction;
   QAction *mpOptionsAction;
@@ -407,9 +415,6 @@ private:
   QAction *mpDiagramWindowAction;
   QAction *mpClearPlotWindowAction;
   QAction *mpExportVariablesAction;
-  // Other Actions
-  QAction *mpExportAsImageAction;
-  QAction *mpExportToClipboardAction;
   // TLM Simulation Action
   QAction *mpSimulationParamsAction;
   QAction *mpFetchInterfaceDataAction;
@@ -448,6 +453,10 @@ private:
   QHash<QString, TransformationsWidget*> mTransformationsWidgetHash;
 public slots:
   void showMessagesBrowser();
+  void switchToWelcomePerspectiveSlot();
+  void switchToModelingPerspectiveSlot();
+  void switchToPlottingPerspectiveSlot();
+  void switchToAlgorithmicDebuggingPerspectiveSlot();
   void showSearchBrowser();
   void createNewModelicaClass();
   void openModelicaFile();
@@ -493,6 +502,7 @@ public slots:
   void importModelFMU();
   void importFMUModelDescription();
   void exportEncryptedPackage();
+  void exportReadonlyPackage();
   void exportModelXML();
   void exportModelFigaro();
   void showOpenModelicaCommandPrompt();
@@ -511,7 +521,6 @@ public slots:
   void openConfigurationOptions();
   void openUsersGuide();
   void openUsersGuidePdf();
-  void openUsersGuideOldPdf();
   void openSystemDocumentation();
   void openOpenModelicaScriptingDocumentation();
   void openModelicaDocumentation();
@@ -533,10 +542,6 @@ private slots:
   void documentationDockWidgetVisibilityChanged(bool visible);
   void threeDViewerDockWidgetVisibilityChanged(bool visible);
   void autoSave();
-  void switchToWelcomePerspectiveSlot();
-  void switchToModelingPerspectiveSlot();
-  void switchToPlottingPerspectiveSlot();
-  void switchToAlgorithmicDebuggingPerspectiveSlot();
   void showDebugConfigurationsDialog();
   void showAttachToProcessDialog();
   void createGitRepository();
@@ -559,9 +564,9 @@ private:
   void tileSubWindows(QMdiArea *pMdiArea, bool horizontally);
   void fetchInterfaceDataHelper(LibraryTreeItem *pLibraryTreeItem, QString singleModel=QString());
 protected:
-  virtual void dragEnterEvent(QDragEnterEvent *event);
-  virtual void dragMoveEvent(QDragMoveEvent *event);
-  virtual void dropEvent(QDropEvent *event);
+  virtual void dragEnterEvent(QDragEnterEvent *event) override;
+  virtual void dragMoveEvent(QDragMoveEvent *event) override;
+  virtual void dropEvent(QDropEvent *event) override;
 };
 
 class AboutOMEditDialog : public QDialog
