@@ -65,6 +65,7 @@ public:
   static QString xmlFileTypes;
   static QString infoXmlFileTypes;
   static QString matFileTypes;
+  static QString csvFileTypes;
   static QString omResultFileTypes;
   static QString exeFileTypes;
   static QString txtFileTypes;
@@ -75,9 +76,11 @@ public:
   static int treeIndentation;
   static QSize iconSize;
   static int tabWidth;
+  static qreal minimumTextFontSize;
   static QString modelicaComponentFormat;
   static QString modelicaFileFormat;
   static QString busConnectorFormat;
+  static QString cutCopyPasteFormat;
   static qreal shapesStrokeWidth;
   static int headingFontSize;
   static QString ModelicaSimulationOutputFormats;
@@ -96,6 +99,7 @@ public:
   static QString structuredOutput;
   static QString textOutput;
   static QString utf8;
+  static const char * const fmuPlatformNamePropertyId;
   static QFontInfo systemFontInfo;
   static QFontInfo monospacedFontInfo;
   static QString toolsOptionsPath;
@@ -158,6 +162,7 @@ public:
   static QString parameters;
   static QString inputs;
   static QString name;
+  static QString startScript;
   static QString comment;
   static QString path;
   static QString type;
@@ -171,8 +176,11 @@ public:
   static QString checkAllModelsTip;
   static QString instantiateModel;
   static QString instantiateModelTip;
-  static QString exportFMU;
+  static QString FMU;
+  static QString exportt;
   static QString exportFMUTip;
+  static QString exportReadonlyPackage;
+  static QString exportRealonlyPackageTip;
   static QString exportEncryptedPackage;
   static QString exportEncryptedPackageTip;
   static QString importFMU;
@@ -361,6 +369,7 @@ public:
   static QString addSystem;
   static QString addSystemTip;
   static QString addSubModel;
+  static QString addExternalTLMModel;
   static QString addSubModelTip;
   static QString addBus;
   static QString addBusTip;
@@ -382,10 +391,10 @@ public:
   static QString moveDown;
   static QString fixErrorsManually;
   static QString revertToLastCorrectVersion;
-  static QString OMCCommandLineOptions;
-  static QString OMCCommandLineOptionsTip;
+  static QString translationFlagsTip;
   static QString saveExperimentAnnotation;
   static QString saveOpenModelicaSimulationFlagsAnnotation;
+  static QString saveOpenModelicaCommandLineOptionsAnnotation;
   static QString item;
   static QString bold;
   static QString italic;
@@ -407,6 +416,7 @@ public:
   static QString terminateInstantiationTip;
   static QString archivedSimulations;
   static QString systemSimulationInformation;
+  static QString translationFlags;
 };
 
 class GUIMessages : public QObject
@@ -430,6 +440,7 @@ public:
     ERROR_IN_TEXT,
     REVERT_PREVIOUS_OR_FIX_ERRORS_MANUALLY,
     NO_OPENMODELICA_KEYWORDS,
+    UNABLE_TO_CREATE_CLASS,
     UNABLE_TO_LOAD_FILE,
     UNABLE_TO_OPEN_FILE,
     UNABLE_TO_SAVE_FILE,
@@ -449,10 +460,12 @@ public:
     MAKE_REPLACEABLE_IF_PARTIAL,
     INNER_MODEL_NAME_CHANGED,
     FMU_GENERATED,
+    FMU_MOVE_FAILED,
     FMU_EMPTY_PLATFORMS,
     XML_GENERATED,
     FIGARO_GENERATED,
     ENCRYPTED_PACKAGE_GENERATED,
+    READONLY_PACKAGE_GENERATED,
     UNLOAD_CLASS_MSG,
     DELETE_CLASS_MSG,
     UNLOAD_TEXT_FILE_MSG,
@@ -477,7 +490,8 @@ public:
     INVALID_TRANSITION_CONDITION,
     MULTIPLE_DECLARATIONS_COMPONENT,
     GDB_ERROR,
-    INVALID_INSTANCE_NAME
+    INVALID_INSTANCE_NAME,
+    ENTER_SCRIPT
   };
 
   static QString getMessage(int type);
