@@ -1,7 +1,7 @@
 /*
  * This file is part of OpenModelica.
  *
- * Copyright (c) 1998-2014, Open Source Modelica Consortium (OSMC),
+ * Copyright (c) 1998-CurrentYear, Open Source Modelica Consortium (OSMC),
  * c/o Linköpings universitet, Department of Computer and Information Science,
  * SE-58183 Linköping, Sweden.
  *
@@ -34,8 +34,10 @@
 
 #include "Utilities.h"
 #include "Helper.h"
+#include "StringHandler.h"
+#include "OMC/OMCProxy.h"
+#include "Modeling/ItemDelegate.h"
 #include "Editors/BaseEditor.h"
-#include "Modeling/LibraryTreeWidget.h"
 
 #include <QApplication>
 #include <QDesktopWidget>
@@ -44,6 +46,8 @@
 #include <QPainter>
 #include <QColorDialog>
 #include <QXmlSchema>
+#include <QXmlSchemaValidator>
+#include <QDir>
 
 SplashScreen *SplashScreen::mpInstance = 0;
 
@@ -61,6 +65,9 @@ TreeSearchFilters::TreeSearchFilters(QWidget *pParent)
   // create the filter text box
   mpFilterTextBox = new QLineEdit;
   mpFilterTextBox->installEventFilter(this);
+  // filter timer
+  mpFilterTimer = new QTimer;
+  mpFilterTimer->setSingleShot(true);
   // show hide button
   mpShowHideButton = new QToolButton;
   QString text = tr("Show/hide filters");
@@ -235,17 +242,22 @@ void Label::setText(const QString &text)
 {
   mText = text;
   setToolTip(text);
-  QLabel::setText(text);
+  QLabel::setText(elidedText());
+}
+
+QString Label::elidedText() const
+{
+  if (mElideMode != Qt::ElideNone) {
+    return fontMetrics().elidedText(mText, mElideMode, size().width());
+  } else {
+    return mText;
+  }
 }
 
 void Label::resizeEvent(QResizeEvent *event)
 {
-  if (mElideMode != Qt::ElideNone) {
-    QFontMetrics fm(fontMetrics());
-    QString str = fm.elidedText(mText, mElideMode, event->size().width());
-    QLabel::setText(str);
-  }
   QLabel::resizeEvent(event);
+  QLabel::setText(elidedText());
 }
 
 FixedCheckBox::FixedCheckBox(QWidget *parent)
@@ -414,6 +426,13 @@ void CodeColorsWidget::pickColor()
   emit colorUpdated();
 }
 
+QString Utilities::escapeForHtmlNonSecure(const QString &str)
+{
+  return QString(str)
+     .replace("& ", "&amp;") // should be the first replacement
+     .replace("< ", "&lt;");
+}
+
 /*!
  * \brief Utilities::tempDirectory
  * Returns the application temporary directory.
@@ -450,7 +469,7 @@ QSettings* Utilities::getApplicationSettings()
   if (!init) {
     init = 1;
     pSettings = new QSettings(QSettings::IniFormat, QSettings::UserScope, Helper::organization, Helper::application);
-    pSettings->setIniCodec(Helper::utf8.toStdString().data());
+    pSettings->setIniCodec(Helper::utf8.toUtf8().constData());
   }
   return pSettings;
 }
@@ -496,6 +515,58 @@ qreal Utilities::convertUnit(qreal value, qreal offset, qreal scaleFactor)
   return (value - offset) / scaleFactor;
 }
 
+/*!
+ * \brief Utilities::isValueLiteralConstant
+ * \param value
+ * \return
+ */
+bool Utilities::isValueLiteralConstant(QString value)
+{
+  bool ok = true;
+  value.toDouble(&ok);
+  if (ok) return true;
+
+  QStringList valuesArray = StringHandler::removeFirstLastCurlBrackets(value).split(",");
+  foreach (QString valueElement, valuesArray) {
+    valueElement.toDouble(&ok);
+  }
+  return ok;
+}
+
+/*!
+ * \brief Utilities::arrayExpressionUnitConversion
+ * If the expression is like an array of constants see ticket:4840
+ * \param pOMCProxy
+ * \param modifierValue
+ * \param fromUnit
+ * \param toUnit
+ * \return
+ */
+QString Utilities::arrayExpressionUnitConversion(OMCProxy *pOMCProxy, QString modifierValue, QString fromUnit, QString toUnit)
+{
+  QStringList modifierValuesArray = StringHandler::removeFirstLastCurlBrackets(modifierValue).split(",");
+  QStringList modifierConvertedValuesArray;
+  OMCInterface::convertUnits_res convertUnit;
+  int i = 0;
+  bool ok = true;
+  foreach (QString modifierValueArrayElement, modifierValuesArray) {
+    qreal modifierRealValueArrayElement = modifierValueArrayElement.toDouble(&ok);
+    if (ok) {
+      if (i == 0) {
+        convertUnit = pOMCProxy->convertUnits(fromUnit, toUnit);
+      }
+      if (convertUnit.unitsCompatible) {
+        modifierRealValueArrayElement = Utilities::convertUnit(modifierRealValueArrayElement, convertUnit.offset, convertUnit.scaleFactor);
+        modifierConvertedValuesArray.append(QString::number(modifierRealValueArrayElement));
+      }
+    }
+  }
+  if (ok) {
+    modifierValue = QString("{%1}").arg(modifierConvertedValuesArray.join(","));
+  }
+  return modifierValue;
+}
+
 Label* Utilities::getHeadingLabel(QString heading)
 {
   Label *pHeadingLabel = new Label(heading);
@@ -1004,3 +1075,40 @@ QList<QPointF> Utilities::liangBarskyClipper(float xmin, float ymin, float xmax,
 //  qDebug() << x1 << y1 << xn1 << yn1 << x2 << y2 << xn2 << yn2;
   return QList<QPointF>() << QPointF(xn1, yn1) << QPointF(xn2, yn2);
 }
+
+/*!
+ * \brief Utilities::removeDirectoryRecursivly
+ * Removes the directory recursively.
+ * \param path
+ */
+void Utilities::removeDirectoryRecursivly(QString path)
+{
+  QFileInfo fileInfo(path);
+  if (fileInfo.isDir()) {
+    QDir dir(path);
+    QStringList filesList = dir.entryList(QDir::AllDirs | QDir::Files | QDir::NoSymLinks |
+                                          QDir::NoDotAndDotDot | QDir::Writable | QDir::CaseSensitive);
+    for (int i = 0 ; i < filesList.count() ; ++i) {
+      removeDirectoryRecursivly(QString("%1/%2").arg(path, filesList.at(i)));
+    }
+    QDir().rmdir(path);
+  } else {
+    QFile::remove(path);
+  }
+}
+
+/*!
+ * \brief Utilities::mapToCoOrdinateSystem
+ * If you have numbers x in the range [a,b] and you want to transform them to numbers y in the range [c,d].\n
+ * y = ((x−a) * ((d−c)/(b−a))) + c
+ * \param value
+ * \param startA
+ * \param endA
+ * \param startB
+ * \param endB
+ * \return
+ */
+qreal Utilities::mapToCoOrdinateSystem(qreal value, qreal startA, qreal endA, qreal startB, qreal endB)
+{
+  return ((value - startA) * ((endB - startB) / (endA - startA))) + startB;
+}
