@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -34,7 +34,6 @@
 
 #include "Component.h"
 #include "ComRef.h"
-#include "Option.h"
 #include "ResultWriter.h"
 #include "Variable.h"
 #include <fmilib.h>
@@ -44,14 +43,14 @@
 #include <unordered_map>
 #include <vector>
 
-namespace oms3
+namespace oms
 {
   class ComponentFMUME : public Component
   {
   public:
     ~ComponentFMUME();
 
-    static Component* NewComponent(const oms3::ComRef& cref, System* parentSystem, const std::string& fmuPath);
+    static Component* NewComponent(const oms::ComRef& cref, System* parentSystem, const std::string& fmuPath);
     static Component* NewComponent(const pugi::xml_node& node, System* parentSystem);
     const FMUInfo* getFMUInfo() const {return &(this->fmuInfo);}
 
@@ -64,6 +63,8 @@ namespace oms3
     oms_status_enu_t initializeDependencyGraph_initialUnknowns();
     oms_status_enu_t initializeDependencyGraph_outputs();
 
+    Variable* getVariable(const ComRef& cref);
+
     oms_status_enu_t getBoolean(const ComRef& cref, bool& value);
     oms_status_enu_t getBoolean(const fmi2_value_reference_t& vr, bool& value);
     oms_status_enu_t getInteger(const ComRef& cref, int& value);
@@ -92,6 +93,10 @@ namespace oms3
     fmi2_import_t* getFMU() {return fmu;}
     fmi2_event_info_t* getEventInfo() {return &eventInfo;}
 
+    bool getCanGetAndSetState() {return getFMUInfo()->getCanGetAndSetFMUstate();}
+
+    oms_status_enu_t setFaultInjection(const ComRef& signal, oms_fault_type_enu_t faultType, double faultValue);
+
   protected:
     ComponentFMUME(const ComRef& cref, System* parentSystem, const std::string& fmuPath);
 
@@ -104,7 +109,6 @@ namespace oms3
     fmi2_callback_functions_t callbackFunctions;
     fmi_import_context_t* context = NULL;
     fmi2_import_t* fmu = NULL;
-    std::string tempDir;
 
     fmi2_event_info_t eventInfo;
     size_t nContinuousStates;
@@ -118,11 +122,13 @@ namespace oms3
     std::vector<Variable> parameters;
     std::vector<bool> exportVariables;
 
-    std::map<std::string, Option<double>> realParameters;
-    std::map<std::string, Option<int>> integerParameters;
-    std::map<std::string, Option<bool>> booleanParameters;
+    std::map<ComRef, double> realStartValues;  ///< parameters and start values defined before instantiating the FMU
+    std::map<ComRef, int> integerStartValues;  ///< parameters and start values defined before instantiating the FMU
+    std::map<ComRef, bool> booleanStartValues; ///< parameters and start values defined before instantiating the FMU
 
     std::unordered_map<unsigned int /*result file var ID*/, unsigned int /*allVariables ID*/> resultFileMapping;
+
+    std::map<fmi2_value_reference_t, oms_fault_type_t> fib;  ///< fault injection blocks
   };
 }
 
