@@ -21,7 +21,7 @@
  * GNU version 3 is obtained from: http://www.gnu.org/copyleft/gpl.html.
  *
  * This program is distributed WITHOUT ANY WARRANTY; without
- * even the implied warranty of  MERCHANTABILITY or FITNESS
+ * even the implied warranty of MERCHANTABILITY or FITNESS
  * FOR A PARTICULAR PURPOSE, EXCEPT AS EXPRESSLY SET FORTH
  * IN THE BY RECIPIENT SELECTED SUBSIDIARY LICENSE CONDITIONS OF OSMC-PL.
  *
@@ -35,7 +35,7 @@
 #include "Model.h"
 #include "ssd/Tags.h"
 #include "System.h"
-#include <OMSBoost.h>
+#include <OMSFileSystem.h>
 #include <RegEx.h>
 
 oms::ComponentTable::ComponentTable(const ComRef& cref, System* parentSystem, const std::string& path)
@@ -71,14 +71,16 @@ oms::Component* oms::ComponentTable::NewComponent(const oms::ComRef& cref, oms::
   if (path.length() > 4)
     extension = path.substr(path.length() - 4);
 
-  boost::filesystem::path temp_root(parentSystem->getModel()->getTempDirectory());
-  boost::filesystem::path relPath = boost::filesystem::path("resources") / (std::string(cref) + extension);
-  boost::filesystem::path absPath = temp_root / relPath;
+  filesystem::path temp_root(parentSystem->getModel()->getTempDirectory());
+  filesystem::path relPath = parentSystem->copyResources() ? (filesystem::path("resources") / (parentSystem->getUniqueID() + "_" + std::string(cref) + extension)) : filesystem::path(path);
+  filesystem::path absPath = temp_root / relPath;
 
-  ComponentTable* component = new ComponentTable(cref, parentSystem, "resources/" + std::string(cref) + extension);
+  ComponentTable* component = new ComponentTable(cref, parentSystem, relPath.string());
 
+  // Copy the resource to the temp directory of the model? We don't want have
+  // do copy resources if importing an SSP or snapshot.
   if (parentSystem->copyResources())
-    boost::filesystem::copy_file(boost::filesystem::path(path), absPath, boost::filesystem::copy_option::overwrite_if_exists);
+    oms_copy_file(filesystem::path(path), absPath);
 
   component->resultReader = ResultReader::newReader(path.c_str());
   if (!component->resultReader)
@@ -92,7 +94,7 @@ oms::Component* oms::ComponentTable::NewComponent(const oms::ComRef& cref, oms::
   int size = 1 + component->resultReader->getAllSignals().size();
   for (auto const &signal : component->resultReader->getAllSignals())
   {
-    component->exportSeries[component->getFullCref() + ComRef(signal)] = true;
+    component->exportSeries[ComRef(signal)] = true;
     component->connectors.back() = new Connector(oms_causality_output, oms_signal_type_real, ComRef(signal), i++/(double)size);
     component->connectors.push_back(NULL);
   }
@@ -123,6 +125,9 @@ oms::Component* oms::ComponentTable::NewComponent(const pugi::xml_node& node, om
       delete connector;
   component->connectors.clear();
 
+  for (auto const &signal : component->resultReader->getAllSignals())
+    component->exportSeries[component->getFullCref() + ComRef(signal)] = false;
+
   for(pugi::xml_node_iterator it = node.begin(); it != node.end(); ++it)
   {
     std::string name = it->name();
@@ -130,7 +135,10 @@ oms::Component* oms::ComponentTable::NewComponent(const pugi::xml_node& node, om
     {
       // import connectors
       for(pugi::xml_node_iterator itConnectors = (*it).begin(); itConnectors != (*it).end(); ++itConnectors)
+      {
         component->connectors.push_back(oms::Connector::NewConnector(*itConnectors));
+        component->exportSeries[component->connectors.back()->getName()] = true;
+      }
     }
     else if(name == oms::ssd::ssd_element_geometry)
     {
@@ -204,22 +212,43 @@ oms_status_enu_t oms::ComponentTable::getReal(const oms::ComRef& cref, double& v
     }
   }
 
-  logError("out of range (cref=" + std::string(cref) + ", time=" + std::to_string(time) + ")");
   value = 0.0;
-  return oms_status_error;
+  return logError("out of range (cref=" + std::string(cref) + ", time=" + std::to_string(time) + ")");
+}
+
+oms_status_enu_t oms::ComponentTable::getRealOutputDerivative(const ComRef& cref, SignalDerivative& value)
+{
+  if (!resultReader)
+    logError("the table isn't initialized properly");
+
+  if (series.find(cref) == series.end())
+    series[cref] = resultReader->getSeries(cref.c_str());
+
+  for (int i=1; i<series[cref]->length; ++i)
+  {
+    if ((series[cref]->time[i-1] <= time && series[cref]->time[i] > time) ||
+        (series[cref]->time[i] == time && i == series[cref]->length-1))
+    {
+      double m = (series[cref]->value[i] - series[cref]->value[i-1]) / (series[cref]->time[i] - series[cref]->time[i-1]);
+      value = SignalDerivative(m);
+      return oms_status_ok;
+    }
+  }
+
+  value = SignalDerivative();
+  return logError("out of range (cref=" + std::string(cref) + ", time=" + std::to_string(time) + ")");
 }
 
 oms_status_enu_t oms::ComponentTable::registerSignalsForResultFile(ResultWriter& resultFile)
 {
   resultFileMapping.clear();
 
-  for (unsigned int i=0; i<resultReader->getAllSignals().size(); ++i)
+  for (unsigned int i=0; i<connectors.size(); ++i)
   {
-    if (!exportSeries[ComRef(resultReader->getAllSignals()[i])])
+    if (!connectors[i] || !exportSeries[connectors[i]->getName()])
       continue;
 
-    std::string name = std::string(getFullCref()) + "." + resultReader->getAllSignals()[i];
-
+    std::string name = std::string(getFullCref() + connectors[i]->getName());
     unsigned int ID = resultFile.addSignal(name, "lookup table", SignalType_REAL);
     resultFileMapping[ID] = i;
   }
@@ -232,7 +261,7 @@ oms_status_enu_t oms::ComponentTable::updateSignals(ResultWriter& resultWriter)
   for (auto const &it : resultFileMapping)
   {
     unsigned int ID = it.first;
-    const std::string& var = resultReader->getAllSignals()[it.second];
+    const ComRef& var = connectors[it.second]->getName();
     SignalValue_t value;
 
     if (oms_status_ok != getReal(var, value.realValue))
@@ -252,10 +281,7 @@ oms_status_enu_t oms::ComponentTable::addSignalsToResults(const char* regex)
       continue;
 
     if (regex_match(std::string(x.first), exp))
-    {
-      //logInfo("added \"" + std::string(x.first) + "\" to results");
       x.second = true;
-    }
   }
 
   return oms_status_ok;
@@ -270,11 +296,25 @@ oms_status_enu_t oms::ComponentTable::removeSignalsFromResults(const char* regex
       continue;
 
     if (regex_match(std::string(x.first), exp))
-    {
-      //logInfo("removed \"" + std::string(x.first) + "\" from results");
       x.second = false;
-    }
   }
 
   return oms_status_ok;
 }
+
+oms_status_enu_t oms::ComponentTable::saveState()
+{
+  storedTime = time;
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentTable::freeState()
+{
+  return oms_status_ok;
+}
+
+oms_status_enu_t oms::ComponentTable::restoreState()
+{
+  time = storedTime;
+  return oms_status_ok;
+}
